<?php
namespace Fiscal\Logic\NotaFiscal;


use Fiscal\Entities\Notafiscal;
use Setforms2\Runtime\Helper\Procedure;
use Setforms2\Runtime\Helper\Procedure\Param;


/**
 * Created by JetBrains PhpStorm.
 * User: paulopereira
 * Date: 13/11/12
 * Time: 11:13
 * To change this template use File | Settings | File Templates.
 */
class ConferenciaErros
{

    private static $avisosNF = array(
        "AV12", // Data de Entrada da nota fiscal difere da Data Programada na AF.
        "AV01", // Fornecedor da nota fiscal difere do fornecedor da AF.
        "AV15", // Data de vencimento da nota fiscal difere da data programada na AF.
        "AV17", // Data de Vencimento informada anterior a data de processamento.
    );
    private static $errosNF  = array(
        "ER03", // Total de vencimentos difere valor total nota fiscal.
        "ER04", // Ã�tem com valor unitÃ¡rio igual a zero.
    );

    private static $avisosItemproduto = array(
        "AV09", // Item com cÃ³digo da situaÃ§Ã£o tributÃ¡ria invÃ¡lido.
        "AV10", // Item com cÃ³digo fiscal de operaÃ§Ã£o invÃ¡lido.
        "AV11", // Quantidade de produto informada difere da quantidade prevista na AF.
        "AV13", // Valor unitÃ¡rio do produto informado difere do valor unitÃ¡rio previsto na AF.
        "AV16", // Item informado na nota fiscal difere do produto programado na AF.
    );

    private static $avisosImposnftot = array(
        "AV07", // Valor de %s informado diferente da tabela.
        "AV08", // %s da NF difere da alÃ­quota do fornecedor.
    );
    private static $errosImposnftot  = array(
        "ER01", // %s da NF invÃ¡lido.
        "ER05", // Soma das entradas sem nota fiscal difere do valor total da nota.
        "ER02", // Total nota fiscal difere soma dos itens.
        "ER03", // Total de vencimentos difere valor total nota fiscal.
    );

    private  static $avisosItemproimpos = array(
        "AV02", // Percentual de %s da tabela difere do percentual informado.
        "AV04", // Imposto do item da entrada com cÃ³digo fiscal invÃ¡lido.
        "AV05", // Imposto do item com situaÃ§Ã£o tributÃ¡ria(CST) invÃ¡lida.
        "AV06", // Imposto do item com cÃ³d. fiscal operaÃ§Ã£o - CFOp invÃ¡lido.
        "AV03", // Percentual %s do item difere do valor %s do Item.
    );

    /**
     * Informa que determinada ocorrencia (erro ou aviso) foi identificado e precisa ser notificado.
     */
    const ID_ERRO_HABILITADO   = 1;
    /**
     * Informa que determinada ocorrencia nao precisa ser notificada.
     */
    const ID_ERRO_DESABILITADO = 0;


    /**
     * Executa a procedure de conferencia de erros da importacao da Nota Fiscal. Este somente armazenara na tabela de
     * Nota Fiscal, nos campos de IDERROS e IDAVISOS o identificador dos possiveis erros acontecidos durante o processo.
     *
     * @static
     *
     * @param Notafiscal $notaFiscal
     * @throws \Exception
     */
    public static function executaProcedureConferenciaDeErros(Notafiscal $notaFiscal)
    {
        try {
            $cdfilial     = $notaFiscal->getFilial()->getCdfilial();
            $nrlanctonf   = $notaFiscal->getNrlanctonf();
            $vrlancamento = $notaFiscal->getVrnotafisc();

            $procedure    = new \Setforms2\Runtime\Helper\Procedure("CONFERE_ERROS_NF");
            self::setParametrosConferenciaErros($procedure, $cdfilial, $nrlanctonf, $vrlancamento);
            $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * "Setta" os parametros da procedure de conferencia de erros.
     *
     * @static
     * @param Procedure $procedure
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $vrlancamento
     * @throws \Exception
     */
    private static function setParametrosConferenciaErros(Procedure $procedure, $cdfilial, $nrlanctonf, $vrlancamento)
    {
        try {
            $procedure->addParam(new Param(
                "P_CDFILIAL",
                Param::PARAM_INPUT,
                $cdfilial,
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_NRLANCTONF",
                Param::PARAM_INPUT,
                $nrlanctonf,
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_VRLANCAMENTO",
                Param::PARAM_INPUT,
                $vrlancamento,
                Param::PARAM_TYPE_STR
            ));
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     */
    public static function confereErrosNotaFiscal(Notafiscal $notaFiscal)
    {
        $idavisos = $notaFiscal->getIdavisos();
        $iderros  = $notaFiscal->getIderros();

//        for ($i = 0; $i < $idavisos != null && count(self::$avisosNF); $i++) {
        for ($i = 0; $i < strlen($idavisos) && $i < count(self::$avisosNF); $i++) {
            if ($idavisos[$i] == self::ID_ERRO_HABILITADO) {
                \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem(self::$avisosNF[$i]);
            }
        }

//        for ($i = 0; $i < $iderros != null && strlen(self::$errosNF); $i++) {
        for ($i = 0; $i < strlen($iderros) && $i < count(self::$errosNF); $i++) {
            if ($iderros[$i] == self::ID_ERRO_HABILITADO) {
                \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem(
                    self::$errosNF[$i],
                    array(),
                    \Geral\Util\Mensagem::ERRO
                );
            }
        }
    }

    /**
     * @static
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     */
    public static function confereErrosItemprodutos(Notafiscal $notaFiscal)
    {
        $itens = \Geral\Entities\Itemproduto::findByCdfilialAndNrlanctonfAndNrorg(
            $notaFiscal->getFilial()->getCdfilial(), $notaFiscal->getNrlanctonf(), \Geral\Util\Ambiente::getNrorg()
        );

        /** @var $item \Geral\Entities\Itemproduto*/
        foreach ($itens as $item) {
            $idavisos = $item->getIdavisositem();
            for ($i = 0; $i < strlen($idavisos); $i++) {
                if ($idavisos[$i] == self::ID_ERRO_HABILITADO) {
                    \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem(self::$avisosItemproduto[$i]);
                }
            }
        }
    }

    /**
     * @static
     * @param \Fiscal\Entities\Notafiscal $notafiscal
     */
    public static function confereErrosImposnftot(Notafiscal $notafiscal)
    {
        $impostos = \Fiscal\Entities\Imposnftot::findByCdfilialAndNrlanctonfAndNrorg(
            $notafiscal->getFilial()->getCdfilial(),
            $notafiscal->getNrlanctonf(),
            \Geral\Util\Ambiente::getNrorg()
        );

        /** @var $imposto \Fiscal\Entities\Imposnftot*/
        foreach ($impostos as $imposto) {
            $idavisostot = $imposto->getIdavisimptt();
            $iderrostot  = $imposto->getIderrosimptt();

            for ($i = 0; $i < strlen($idavisostot); $i++) {
                if ($idavisostot[$i] == self::ID_ERRO_HABILITADO) {
                    \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem(self::$avisosImposnftot[$i]);
                }
            }

            for ($i = 0; $i < strlen($iderrostot); $i++) {
                if ($iderrostot[$i] == self::ID_ERRO_HABILITADO) {
                    \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem(self::$errosImposnftot[$i]);
                }
            }
        }
    }

    /**
     * @static
     * @param \Fiscal\Entities\Notafiscal $notafiscal
     */
    public static function confereErrosItemproimpos(Notafiscal $notafiscal)
    {
        $impostos = \Fiscal\Entities\Itemproimpos::findByCdfilialAndNrlanctonfAndNrorg(
            $notafiscal->getFilial()->getCdfilial(),
            $notafiscal->getNrlanctonf(),
            \Geral\Util\Ambiente::getNrorg()
        );

        /** @var $imposto \Fiscal\Entities\Itemproimpos */
        foreach ($impostos as $imposto) {
            $idavisimpit = $imposto->getIdavisimpit();

            for ($i = 0; $i < strlen($idavisimpit); $i++) {
                if ($idavisimpit[$i] == self::ID_ERRO_HABILITADO) {
                    \Geral\Util\AcumuladorDeMensagens::getInstance()->addMensagem(self::$avisosItemproimpos[$i]);
                }
            }
        }
    }

    /**
     * @static
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     * @throws \Exception
     */
    public static function confereErros(Notafiscal $notaFiscal) {
        self::confereErrosNotaFiscal($notaFiscal);
        self::confereErrosItemprodutos($notaFiscal);
        self::confereErrosImposnftot($notaFiscal);
        self::confereErrosItemproimpos($notaFiscal);
        if (\Geral\Util\AcumuladorDeMensagens::getInstance()->existeMensagemErro()) {
            throw new \Exception(\Geral\Util\AcumuladorDeMensagens::ID_ACUMULADOR_DE_MENSAGENS);
        }
    }

}
