/* Sistemas Integrados TEKNISA
  Documentos Fiscais

Nome: 5550_DFESTPS_GERA_RE.SQL

@C:\Web_575\Sql\End_Conv\sqlserver\5550_DFESTPS_GERA_RE.sql

Descrio:
Rotina para gerao da capa do documento (RE).

P_CDFILIAL      - Cdigo da filial a ser gerado RE.
P_IDENTRSAID    - Identificador de 'E' - Entrada e 'S' - Sada
P_DATARE        - Data da gerao do RE
P_TIPOEXPRE     - Tipo de RE ('F' - Filial e 'C' - Centralizadora)
P_CDALMOXARIFE  - Almoxarifado
P_CDLOCALESTOQ  - Localizao
P_IDERRO        - Parmetro de retorno para indicar se o RE de entrada j foi emitido.

PARA DEBUGAR
-- SET SERVEROUTPUT ON
-- DBMS_OUTPUT.PUT_LINE('xx');
-- RAISERROR( 'Inserindo', 16,1);

Criao:
08/03/2010 - Fabrcio Marcelo da Silva
*/

IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
                    WHERE
	        O.NAME = 'GERA_RE'         AND
	        TYPE = 'P' 			        AND
	        U.UID = O.UID              AND
	        U.NAME = USER)

DROP PROCEDURE GERA_RE
GO

CREATE PROCEDURE GERA_RE @P_CDFILIAL       VARCHAR(04),
                         @P_IDENTRSAID     VARCHAR(01),
                         @P_DATARE         DATETIME,
                         @P_TIPOEXPRE      VARCHAR(01),
                         @P_CDALMOXARIFE   VARCHAR(06),
                         @P_CDLOCALESTOQ   VARCHAR(06),
                         @P_IDERRO         VARCHAR(01)OUTPUT
AS
   DECLARE @V_DTPROCESSA       DATETIME;
   DECLARE @V_IDREINICIANUMRE  CHAR(1);  -- Identificador reiniciar nmerao de RE
   DECLARE @V_IDPARAMEMISRE    CHAR(1);  -- Identificador para considerar parametrizao para emisso do RE
   DECLARE @V_CDPARAMEMISRE    VARCHAR(10);
   DECLARE @V_IDCAPACORRES     CHAR(1);  -- Identificador se possui capa correspondente conforme parametrizao

   DECLARE @V_ANTNUMRE         VARCHAR(20);
   DECLARE @V_NUMERORE         VARCHAR(10);
   DECLARE @V_PRXNUMRE         VARCHAR(20);
   DECLARE @V_NRREPOST         VARCHAR(10);

   DECLARE @V_IDSEQCRIADO      CHAR(1);  -- Identificador de sequencial RE criado

   DECLARE @V_CDMODEDOCU       VARCHAR(200);
   DECLARE @V_CDTIPOOPER       VARCHAR(200);
   DECLARE @V_CDFORNECED       VARCHAR(1000);
   DECLARE @V_CDCLIENTE        VARCHAR(1000);

   DECLARE @V_DTFECMES         DATETIME;
   DECLARE @V_NRMAIORRE        VARCHAR(20);

   DECLARE @V_NOVO_CODIGO      VARCHAR(16);


   /* Variveis do cursor C_PARAMEMIS */
   DECLARE @V_C_CDPARAMEMISRE     VARCHAR(10);
   DECLARE @V_C_DSOBPARAMEMISRE   VARCHAR(250);

   /* Variveis do cursor C_MODEDOCU */
   DECLARE @V_C_CDMODDOCFISC      VARCHAR(02);

   /* Variveis do cursor C_TIPOOPER */
   DECLARE @V_C_CDTIPOOPER        VARCHAR(02);

   /* Variveis do cursor C_FORNECED */
   DECLARE @V_C_CDFORNECED        VARCHAR(21);

   /* Variveis do cursor C_CLIENTE */
   DECLARE @V_C_CDCLIENTE         VARCHAR(21);

   /* Variveis cursor C_NOTAFISCALE (Notas fiscais de entrada) */
   DECLARE @V_NE_CDFILIAL         VARCHAR(04);
   DECLARE @V_NE_NRLANCTONF       VARCHAR(10);
   DECLARE @V_NE_CDMODDOCFISC     VARCHAR(02);
   DECLARE @V_NE_CDTIPOOPER       VARCHAR(02);
   DECLARE @V_NE_CDFORNECED       VARCHAR(21);
   DECLARE @V_NE_CDCLIENTE        VARCHAR(21);

   /* Variveis cursor C_NOTAFISCALS (Notas fiscais de sada) */
   DECLARE @V_NS_CDFILIAL         VARCHAR(04);
   DECLARE @V_NS_NRLANCTONF       VARCHAR(10);
   DECLARE @V_NS_CDMODDOCFISC     VARCHAR(02);
   DECLARE @V_NS_CDTIPOOPER       VARCHAR(02);
   DECLARE @V_NS_CDFORNECED       VARCHAR(21);
   DECLARE @V_NS_CDCLIENTE        VARCHAR(21);

   /* Variveis cursor C_NOTAFISCALSC (Notas fiscais de sada - RE Centralizadora) */
   DECLARE @V_NSC_CDFILIAL        VARCHAR(04);
   DECLARE @V_NSC_NRLANCTONF      VARCHAR(10);
   DECLARE @V_NSC_CDMODDOCFISC    VARCHAR(02);
   DECLARE @V_NSC_CDTIPOOPER      VARCHAR(02);
   DECLARE @V_NSC_CDFORNECED      VARCHAR(21);
   DECLARE @V_NSC_CDCLIENTE       VARCHAR(21);

   DECLARE @V_REGERADOE           VARCHAR(04);

   DECLARE @V_CDTIPOFATUR         VARCHAR(03);
   DECLARE @V_CDINSCESTA          VARCHAR(16);
   DECLARE @V_CDEMPRESA           VARCHAR(02);

   DECLARE @V_PCDTIPOCONT         VARCHAR(03);
   DECLARE @V_PCDINSCESTA         VARCHAR(16);
   DECLARE @V_PCDEMPRESA          VARCHAR(02);

   DECLARE @V_NUMRENANO           VARCHAR(20);
   DECLARE @V_NUMRENPER           VARCHAR(20);
   DECLARE @V_DTFECMESANT         DATETIME;

   DECLARE @V_REGERADO            VARCHAR(01);
   DECLARE @V_NRREAUX             VARCHAR(20);
   DECLARE @V_CDREGRA             VARCHAR(10);

   DECLARE @V_NUMERORED           VARCHAR(10);   
BEGIN
   SET @P_IDERRO = 'N';
   SET @V_NOVO_CODIGO = NULL;
   SET @V_IDCAPACORRES = 'S';
   SET @V_IDSEQCRIADO = 'N';

   /* Verifica a data de processamento da filial, se ser reiniciado a nmerao do RE
      e se a filial considera parametrizao para emisso do RE */
   SET @V_DTPROCESSA = NULL;
   SET @V_IDREINICIANUMRE = 'N';
   SET @V_IDPARAMEMISRE = 'N';

   SELECT @V_DTPROCESSA = MAX(PF.DTPROCESSA),
          @V_IDREINICIANUMRE = MAX(PF.IDREINICIANUMRE),
          @V_IDPARAMEMISRE =  CASE MAX(PF.IDPARAMEMISREFI) WHEN 'N' THEN MAX(P.IDPARAMEMISRE) ELSE 'S' END
     FROM PARAMFILIAL PF, PARAMGERAL P
    WHERE PF.CDFILIAL = @P_CDFILIAL
      AND P.CDPARGERAL = '0001';

   /* Verifica se existe parametrizao para emisso do RE que atenda a filial
      a ser gerada a capa de documento (RE) */
   SET @V_CDPARAMEMISRE = NULL;

   SET @V_CDTIPOFATUR = NULL;
   SET @V_CDINSCESTA = NULL;
   SET @V_CDEMPRESA = NULL;

   SELECT @V_CDTIPOFATUR = MAX(PF.CDTIPOFATUR), @V_CDINSCESTA = MAX(F.CDINSCESTA),
          @V_CDEMPRESA = MAX(F.CDEMPRESA)
     FROM FILIAL F, PARAMFILIAL PF
    WHERE F.CDFILIAL = @P_CDFILIAL
      AND PF.CDFILIAL = @P_CDFILIAL;

   SELECT @V_CDPARAMEMISRE = MAX(CDPARAMEMISRE)
     FROM PARAMEMISSAORE P
    WHERE P.CDTIPOFATEMISRE = @V_CDTIPOFATUR
      AND P.CDEMPRESA = @V_CDEMPRESA
      AND P.CDINSCESTA = @V_CDINSCESTA
      AND P.IDENTSAIDEMISRE = @P_IDENTRSAID
      AND P.IDSITUQUEBRARE = 'A';

   IF @V_CDPARAMEMISRE IS NULL BEGIN
      SELECT @V_CDPARAMEMISRE = MAX(CDPARAMEMISRE)
        FROM PARAMEMISSAORE P
       WHERE P.CDTIPOFATEMISRE = @V_CDTIPOFATUR
         AND P.CDEMPRESA = @V_CDEMPRESA
         AND P.CDINSCESTA IS NULL
         AND P.IDENTSAIDEMISRE = @P_IDENTRSAID
         AND P.IDSITUQUEBRARE = 'A';

      IF @V_CDPARAMEMISRE IS NULL BEGIN
         SELECT @V_CDPARAMEMISRE = MAX(CDPARAMEMISRE)
           FROM PARAMEMISSAORE P
          WHERE P.CDTIPOFATEMISRE IS NULL
            AND P.CDEMPRESA = @V_CDEMPRESA
            AND P.CDINSCESTA = @V_CDINSCESTA
            AND P.IDENTSAIDEMISRE = @P_IDENTRSAID
            AND P.IDSITUQUEBRARE = 'A';

         IF @V_CDPARAMEMISRE IS NULL BEGIN
            SELECT @V_CDPARAMEMISRE = MAX(CDPARAMEMISRE)
              FROM PARAMEMISSAORE P
             WHERE P.CDTIPOFATEMISRE IS NULL
               AND P.CDEMPRESA = @V_CDEMPRESA
               AND P.CDINSCESTA IS NULL
               AND P.IDENTSAIDEMISRE = @P_IDENTRSAID
               AND P.IDSITUQUEBRARE = 'A';

            IF @V_CDPARAMEMISRE IS NULL BEGIN
               SET @V_PCDTIPOCONT = NULL;
               SET @V_PCDEMPRESA  = @V_CDEMPRESA;
               SET @V_PCDINSCESTA = NULL;
            END
            ELSE BEGIN
               SET @V_PCDTIPOCONT = 'T';
               SET @V_PCDEMPRESA  = @V_CDEMPRESA;
               SET @V_PCDINSCESTA = 'T';
            END;
         END
         ELSE BEGIN
            SET @V_PCDTIPOCONT = 'T';
            SET @V_PCDEMPRESA  = @V_CDEMPRESA;
            SET @V_PCDINSCESTA = @V_CDINSCESTA;
         END;
      END
      ELSE BEGIN
         SET @V_PCDTIPOCONT = @V_CDTIPOFATUR;
         SET @V_PCDEMPRESA  = @V_CDEMPRESA;
         SET @V_PCDINSCESTA = 'T';
      END;
   END
   ELSE BEGIN
      SET @V_PCDTIPOCONT = @V_CDTIPOFATUR;
      SET @V_PCDEMPRESA  = @V_CDEMPRESA;
      SET @V_PCDINSCESTA = @V_CDINSCESTA;
   END;

   /* RE de notas fiscais (NORMAL) */
   IF @P_TIPOEXPRE = 'F' BEGIN
      /* Notas fiscais de Entrada */
      IF @P_IDENTRSAID = 'E' BEGIN

         SET @V_REGERADO = 'N';

         /* Verifica se possui algum RE posterior a data do RE gerado */
         SELECT @V_NRREPOST = MIN(NRNUMERNF)
           FROM NUMERORNF
          WHERE CDFILIAL = @P_CDFILIAL
            AND DTNUMERNF > @P_DATARE;

         IF @V_NRREPOST IS NULL BEGIN

            /* Verifica se a data do RE  igual a data de processamento da filial,
               caso seja, ser gerado novamente o RE de notas fiscais */
            IF @V_DTPROCESSA = @P_DATARE BEGIN
               SET @V_NRREAUX = NULL;
               SELECT @V_NRREAUX = MIN(NRSEQCAPADOC)
                 FROM NOTAFISCAL
                WHERE CDFILIRECE = @P_CDFILIAL
                  AND IDENTRSAIDOP = @P_IDENTRSAID
                  AND DTENTRSAID = @P_DATARE
                  AND (IDSISTEMA IN ('EST','DFE') OR IDSISTEMA IS NULL);



               UPDATE NOTAFISCAL
                  SET CDPARAMEMISRE = NULL,
                      NRSEQCAPADOC = NULL
                WHERE CDFILIRECE = @P_CDFILIAL
                  AND IDENTRSAIDOP = @P_IDENTRSAID
                  AND DTENTRSAID = @P_DATARE
                  AND (IDSISTEMA IN ('EST','DFE') OR IDSISTEMA IS NULL);

               UPDATE NOTAFISCAL
                  SET CDPARAMEMISRE = NULL,
                      NRSEQCAPADOC = NULL
                WHERE CDFILIEMIT = @P_CDFILIAL
                  AND IDENTRSAIDOP = 'S'
                  AND DTENTRSAID = @P_DATARE
                  AND (IDSISTEMA IN ('EST','DFS','FAI') OR IDSISTEMA IS NULL);

               DELETE FROM NUMERORNF
                WHERE CDFILIAL = @P_CDFILIAL
                  AND DTNUMERNF = @P_DATARE;

               /* Retorna sequencial do dia ltimo dia gerado, caso seja cancelado o RE do dia */
               SET @V_ANTNUMRE = NULL;
               SELECT @V_ANTNUMRE = RIGHT(REPLICATE('0',20) + MAX(NRNUMERNF),20)
                 FROM NUMERORNF
                WHERE CDFILIAL = @P_CDFILIAL
                  AND DTNUMERNF = (SELECT MAX(DTNUMERNF)
                                     FROM NUMERORNF
                                    WHERE CDFILIAL = @P_CDFILIAL
                                      AND DTNUMERNF <= (@P_DATARE - 1));


               IF @V_ANTNUMRE IS NULL BEGIN
                  IF NOT (@V_NRREAUX IS NULL) BEGIN
                     SET @V_ANTNUMRE = CONVERT(CHAR(4), CONVERT(INT,@V_NRREAUX) - 1);
                  END;
               END;

               IF @V_ANTNUMRE IS NOT NULL BEGIN
                  UPDATE NOVOCODIGO
                     SET NRSEQUENCIAL = @V_ANTNUMRE
                   WHERE CDCONTADOR = 'RENOTAFISCAL' + @P_CDFILIAL;
               END;
            END;
         END
         ELSE BEGIN
            SET @V_NUMERORE = NULL;

            /* Verifica se j possui RE (Capa de documento) gerada para o dia */
            SELECT @V_NUMERORE = MAX(N.NRSEQCAPADOC)
              FROM NOTAFISCAL N
             WHERE N.CDFILIRECE = @P_CDFILIAL
               AND N.IDENTRSAIDOP = 'E'
               AND N.DTENTRSAID = @P_DATARE
               AND (N.IDSISTEMA IN ('EST','DFE') OR N.IDSISTEMA IS NULL)
               AND NOT N.NRSEQCAPADOC IS NULL;

            IF NOT @V_NUMERORE IS NULL BEGIN
               SET @V_CDREGRA = NULL;

               SELECT @V_CDREGRA = MAX(N.CDPARAMEMISRE)
                 FROM NOTAFISCAL N
                WHERE N.CDFILIRECE = @P_CDFILIAL
                  AND N.IDENTRSAIDOP = 'E'
                  AND N.DTENTRSAID = @P_DATARE
                  AND (N.IDSISTEMA IN ('EST','DFE') OR N.IDSISTEMA IS NULL)
                  AND N.NRSEQCAPADOC = @V_NUMERORE;

               DECLARE C_NOTAFISCALE CURSOR FOR
                  SELECT DISTINCT N.CDFILIAL, N.NRLANCTONF, N.CDMODDOCFISC, N.CDTIPOOPER, ISNULL(N.CDFORNECED,''), ISNULL(N.CDCLIENTE,'')
                    FROM NOTAFISCAL N, TIPOOPERAC T,
                         ITEMPRODUTO I LEFT JOIN ALMOXARIFADO A ON (I.CDFILIAL = A.CDFILIAL)         AND
                                                                   (I.CDALMOXARIFE = A.CDALMOXARIFE)
                                       LEFT JOIN LOCALESTOQUE L ON (I.CDFILIAL = L.CDFILIAL)         AND
                                                                   (I.CDALMOXARIFE = L.CDALMOXARIFE) AND
                                                                   (I.CDLOCALESTOQ = L.CDLOCALESTOQ)
                   WHERE N.CDFILIRECE = @P_CDFILIAL
                     AND N.IDENTRSAIDOP = 'E'
                     AND N.DTENTRSAID = @P_DATARE
                     AND (N.IDSISTEMA IN ('EST','DFE') OR N.IDSISTEMA IS NULL)
                     AND N.CDTIPOOPER NOT IN ('10', '09')
                     AND N.NRSEQCAPADOC IS NULL
                     AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                     AND N.CDTIPOOPER = T.CDTIPOOPER
                     AND ISNULL(T.IDCOMPEMISRE, 'F') = 'F'
                     AND I.CDFILIAL = N.CDFILIAL
                     AND I.NRLANCTONF = N.NRLANCTONF
                     AND ((I.CDALMOXARIFE = @P_CDALMOXARIFE) OR (@P_CDALMOXARIFE = 'T'))
                     AND ((I.CDLOCALESTOQ = @P_CDLOCALESTOQ) OR (@P_CDLOCALESTOQ = 'T'));

               OPEN C_NOTAFISCALE;
               FETCH NEXT FROM C_NOTAFISCALE INTO @V_NE_CDFILIAL,   @V_NE_NRLANCTONF, @V_NE_CDMODDOCFISC,
                                                  @V_NE_CDTIPOOPER, @V_NE_CDFORNECED, @V_NE_CDCLIENTE;
               WHILE (@@FETCH_STATUS = 0)
               BEGIN
                  UPDATE NOTAFISCAL
                     SET NRSEQCAPADOC = @V_NUMERORE,
                         CDPARAMEMISRE = @V_CDREGRA
                   WHERE CDFILIAL = @V_NE_CDFILIAL
                     AND NRLANCTONF = @V_NE_NRLANCTONF;

                  FETCH NEXT FROM C_NOTAFISCALE INTO @V_NE_CDFILIAL,   @V_NE_NRLANCTONF, @V_NE_CDMODDOCFISC,
                                                     @V_NE_CDTIPOOPER, @V_NE_CDFORNECED, @V_NE_CDCLIENTE;

               END;

               CLOSE C_NOTAFISCALE;
               DEALLOCATE C_NOTAFISCALE;
            END;
            SET @V_REGERADO = 'S';
         END;

         IF @V_REGERADO = 'N' BEGIN

            /* Verifica a data de fechamento do ms de dezembro */
            SET @V_NUMRENANO = NULL;

            SELECT @V_NUMRENANO = MAX(NRNUMERNF)
              FROM NUMERORNF
             WHERE CDFILIAL = @P_CDFILIAL
               AND DTNUMERNF >= CONVERT(DATETIME, '01/01/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)));


            SET @V_DTFECMESANT = CONVERT(DATETIME, '01/01/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)));

            SELECT @V_DTFECMESANT = DTFECMES + 1
              FROM DIAFECMES
             WHERE CDFILIAL = @P_CDFILIAL
               AND DTFECMES BETWEEN CONVERT(DATETIME, '01/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)-1)) AND CONVERT(DATETIME, '31/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)-1));


            SET @V_NUMRENPER = NULL;

            SELECT @V_NUMRENPER = MAX(NRNUMERNF)
              FROM NUMERORNF
             WHERE CDFILIAL = @P_CDFILIAL
               AND DTNUMERNF BETWEEN @V_DTFECMESANT AND @P_DATARE;

            IF (@V_NUMRENANO IS NULL  AND
                @V_NUMRENPER IS NULL) BEGIN
               SET @V_DTFECMES = CONVERT(DATETIME, '31/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)-1));

               SELECT @V_DTFECMES = MAX(DTFECMES)
                 FROM DIAFECMES
                WHERE CDFILIAL = @P_CDFILIAL
                  AND DTFECMES BETWEEN CONVERT(DATETIME, '01/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)-1)) AND CONVERT(DATETIME, '31/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)-1));
            END
            ELSE BEGIN
               SET @V_DTFECMES = CONVERT(DATETIME, '31/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)));

               SELECT @V_DTFECMES = MAX(DTFECMES)
                 FROM DIAFECMES
                WHERE CDFILIAL = @P_CDFILIAL
                  AND DTFECMES BETWEEN CONVERT(DATETIME, '01/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE))) AND CONVERT(DATETIME, '31/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)));
            END;

            /* Verifica se existe algum RE posterior a data de fechamento do ms de dezembro do ano anterior */
            SET @V_NRMAIORRE = NULL;

            SELECT @V_NRMAIORRE = MAX(NRNUMERNF)
              FROM NUMERORNF
             WHERE CDFILIAL = @P_CDFILIAL
               AND DTNUMERNF <> @P_DATARE
               AND DTNUMERNF > @V_DTFECMES;

            /* Utiliza parametrizao para emisso do RE (Regras) e possui regras */
            IF (@V_IDPARAMEMISRE = 'S' AND NOT @V_CDPARAMEMISRE IS NULL) BEGIN
               /* Regras disponveis para a filial para gerao da capa de documento (RE) */
               DECLARE C_PARAMEMIS CURSOR FOR
                  SELECT CDPARAMEMISRE, DSOBPARAMEMISRE
                    FROM PARAMEMISSAORE
                   WHERE ISNULL(CDTIPOFATEMISRE, 'T') = @V_PCDTIPOCONT
                     AND CDEMPRESA = @V_PCDEMPRESA
                     AND ISNULL(CDINSCESTA, 'T') = @V_PCDINSCESTA
                     AND IDENTSAIDEMISRE = @P_IDENTRSAID
                     AND IDSITUQUEBRARE = 'A'
                   ORDER BY CDPRIOEMISSAORE;
               OPEN C_PARAMEMIS;

               FETCH NEXT FROM C_PARAMEMIS INTO @V_C_CDPARAMEMISRE, @V_C_DSOBPARAMEMISRE;
               WHILE (@@FETCH_STATUS = 0)
               BEGIN
                  SET @V_IDSEQCRIADO = 'N';

                  SET @V_CDMODEDOCU = '';
                  SET @V_CDTIPOOPER = '';
                  SET @V_CDFORNECED = '';
                  SET @V_CDCLIENTE  = '';

                  DECLARE C_MODEDOCU CURSOR FOR
                     SELECT CDMODDOCFISC
                       FROM MODDOCEMISSAORE
                      WHERE CDPARAMEMISRE = @V_C_CDPARAMEMISRE;
                  OPEN C_MODEDOCU;

                  FETCH NEXT FROM C_MODEDOCU INTO @V_C_CDMODDOCFISC;
                  WHILE (@@FETCH_STATUS = 0)
                  BEGIN
                     SET @V_CDMODEDOCU = @V_CDMODEDOCU + @V_C_CDMODDOCFISC + '|';

                     FETCH NEXT FROM C_MODEDOCU INTO @V_C_CDMODDOCFISC;
                  END;
                  CLOSE C_MODEDOCU;
                  DEALLOCATE C_MODEDOCU;

                  DECLARE C_TIPOOPER CURSOR FOR
                     SELECT CDTIPOOPER
                       FROM TIPOPERACEMISRE
                      WHERE CDPARAMEMISRE = @V_C_CDPARAMEMISRE
                        AND IDENTRSAIDOP = @P_IDENTRSAID;
                  OPEN C_TIPOOPER;

                  FETCH NEXT FROM C_TIPOOPER INTO @V_C_CDTIPOOPER;
                  WHILE (@@FETCH_STATUS = 0)
                  BEGIN
                     SET @V_CDTIPOOPER = @V_CDTIPOOPER + @V_C_CDTIPOOPER + '|';
                     FETCH NEXT FROM C_TIPOOPER INTO @V_C_CDTIPOOPER;
                  END;
                  CLOSE C_TIPOOPER;
                  DEALLOCATE C_TIPOOPER;

                  DECLARE C_FORNECED CURSOR FOR
                     SELECT CDFORNECED
                       FROM FORNECEMISSAORE
                      WHERE CDPARAMEMISRE = @V_C_CDPARAMEMISRE;
                  OPEN C_FORNECED;

                  FETCH NEXT FROM C_FORNECED INTO @V_C_CDFORNECED;
                  WHILE (@@FETCH_STATUS = 0)
                  BEGIN
                     SET @V_CDFORNECED = @V_CDFORNECED + @V_C_CDFORNECED + '|';
                     FETCH NEXT FROM C_FORNECED INTO @V_C_CDFORNECED;
                  END;
                  CLOSE C_FORNECED;
                  DEALLOCATE C_FORNECED;

                  DECLARE C_CLIENTE CURSOR FOR
                     SELECT CDCLIENTE
                       FROM CLIENTEMISSAORE
                      WHERE CDPARAMEMISRE = @V_C_CDPARAMEMISRE;
                  OPEN C_CLIENTE;

                  FETCH NEXT FROM C_CLIENTE INTO @V_C_CDCLIENTE;
                  WHILE (@@FETCH_STATUS = 0)
                  BEGIN
                     SET @V_CDCLIENTE = @V_CDCLIENTE + @V_C_CDCLIENTE + '|';
                     FETCH NEXT FROM C_CLIENTE INTO @V_C_CDCLIENTE;
                  END;
                  CLOSE C_CLIENTE;
                  DEALLOCATE C_CLIENTE;

                  /* Notas fiscais de entrada */
                  DECLARE C_NOTAFISCALE CURSOR FOR
                     SELECT DISTINCT N.CDFILIAL, N.NRLANCTONF, N.CDMODDOCFISC, N.CDTIPOOPER, ISNULL(N.CDFORNECED,''), ISNULL(N.CDCLIENTE,'')
                       FROM NOTAFISCAL N, TIPOOPERAC T,
                            ITEMPRODUTO I LEFT JOIN ALMOXARIFADO A ON (I.CDFILIAL = A.CDFILIAL)         AND
                                                                      (I.CDALMOXARIFE = A.CDALMOXARIFE)
                                          LEFT JOIN LOCALESTOQUE L ON (I.CDFILIAL = L.CDFILIAL)         AND
                                                                      (I.CDALMOXARIFE = L.CDALMOXARIFE) AND
                                                                      (I.CDLOCALESTOQ = L.CDLOCALESTOQ)
                      WHERE N.CDFILIRECE = @P_CDFILIAL
                        AND N.IDENTRSAIDOP = 'E'
                        AND N.DTENTRSAID = @P_DATARE
                        AND (N.IDSISTEMA IN ('EST','DFE') OR N.IDSISTEMA IS NULL)
                        AND N.CDTIPOOPER NOT IN ('10', '09')
                        AND N.NRSEQCAPADOC IS NULL
                        AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                        AND N.CDTIPOOPER = T.CDTIPOOPER
                        AND ISNULL(T.IDCOMPEMISRE, 'F') = 'F'
                        AND I.CDFILIAL = N.CDFILIAL
                        AND I.NRLANCTONF = N.NRLANCTONF
                        AND ((I.CDALMOXARIFE = @P_CDALMOXARIFE) OR (@P_CDALMOXARIFE = 'T'))
                        AND ((I.CDLOCALESTOQ = @P_CDLOCALESTOQ) OR (@P_CDLOCALESTOQ = 'T'));

                  OPEN C_NOTAFISCALE;
                  FETCH NEXT FROM C_NOTAFISCALE INTO @V_NE_CDFILIAL,   @V_NE_NRLANCTONF, @V_NE_CDMODDOCFISC,
                                                     @V_NE_CDTIPOOPER, @V_NE_CDFORNECED, @V_NE_CDCLIENTE;
                  WHILE (@@FETCH_STATUS = 0)
                  BEGIN
                     SET @V_IDCAPACORRES = 'S';

                     IF (NOT @V_CDMODEDOCU = '' AND @V_IDCAPACORRES = 'S') BEGIN
                        IF CHARINDEX(@V_NE_CDMODDOCFISC, @V_CDMODEDOCU) = 0 BEGIN
                           SET @V_IDCAPACORRES = 'N';
                        END;
                     END;

                     IF (NOT @V_CDTIPOOPER = '' AND @V_IDCAPACORRES = 'S') BEGIN
                        IF CHARINDEX(@V_NE_CDTIPOOPER, @V_CDTIPOOPER) = 0 BEGIN
                           SET @V_IDCAPACORRES = 'N';
                        END;
                     END;

                     IF (NOT @V_CDFORNECED = '' AND @V_IDCAPACORRES = 'S') OR
                        (NOT @V_CDCLIENTE = '' AND @V_IDCAPACORRES = 'S')  BEGIN
                        IF ((CHARINDEX(@V_NE_CDFORNECED, @V_CDFORNECED) = 0) AND
                           (CHARINDEX(@V_NE_CDCLIENTE, @V_CDCLIENTE) = 0))   BEGIN
                           SET @V_IDCAPACORRES = 'N';
                        END;
                     END;

                     /* Se atende as regras de capa de documento conforme parametrizao (regras) */
                     IF @V_IDCAPACORRES = 'S' BEGIN
                        /* Verifica se j possui RE (Capa de documento) gerada para o dia conforme regras */
                        SET @V_NUMERORE = NULL;

                        SELECT @V_NUMERORE = MAX(N.NRSEQCAPADOC)
                          FROM NOTAFISCAL N
                         WHERE N.CDFILIRECE = @P_CDFILIAL
                           AND N.IDENTRSAIDOP = 'E'
                           AND N.DTENTRSAID = @P_DATARE
                           AND (N.IDSISTEMA IN ('EST','DFE') OR N.IDSISTEMA IS NULL)
                           AND NOT N.NRSEQCAPADOC IS NULL
                           AND N.CDPARAMEMISRE = @V_C_CDPARAMEMISRE;

                        IF @V_NUMERORE IS NULL BEGIN
                           IF @V_IDSEQCRIADO <> 'S' BEGIN
                              IF ((@V_IDREINICIANUMRE = 'S') AND
                                  (@P_DATARE > @V_DTFECMES)   AND
                                  (@V_NRMAIORRE IS NULL))    BEGIN

                                 /* Verifica se j possui RE (Capa de documento) gerada para o dia indepedente de regras */
                                 SET @V_NUMERORED = NULL;

                                 SELECT @V_NUMERORED = MAX(N.NRSEQCAPADOC)
                                   FROM NOTAFISCAL N
                                  WHERE N.CDFILIRECE = @P_CDFILIAL
                                    AND N.IDENTRSAIDOP = 'E'
                                    AND N.DTENTRSAID = @P_DATARE
                                    AND (N.IDSISTEMA IN ('EST','DFE') OR N.IDSISTEMA IS NULL)
                                    AND NOT N.NRSEQCAPADOC IS NULL;

                                 IF @V_NUMERORED IS NULL BEGIN
                                    UPDATE NOVOCODIGO
                                       SET NRSEQUENCIAL = RIGHT(REPLICATE('0',20) + '0',20)
                                     WHERE CDCONTADOR = 'RENOTAFISCAL' + @P_CDFILIAL;
                                 END;    
                              END;

                              SET @V_PRXNUMRE = NULL;

                              SET @V_NOVO_CODIGO = 'RENOTAFISCAL'+@P_CDFILIAL;
                              EXEC NOVO_CODIGO @V_NOVO_CODIGO, @P_SEQUENCIAL = @V_PRXNUMRE OUTPUT, @P_QTDE = 1;
                              SET @V_NUMERORE = SUBSTRING(@V_PRXNUMRE, 11, 10);

                              /* Insere novo nmero de RE */
                              INSERT INTO NUMERORNF
                                (CDFILIAL, DTNUMERNF, IDENTRSAID, IDTIPOOPERNF, NRNUMERNF)
                              VALUES
                                (@P_CDFILIAL, @P_DATARE, @P_IDENTRSAID, @P_IDENTRSAID, @V_NUMERORE);

                              UPDATE PARAMFILIAL
                                 SET NRRENOTAFISC = @V_NUMERORE
                               WHERE CDFILIAL = @P_CDFILIAL;

                              SET @V_IDSEQCRIADO = 'S';
                           END;
                        END;
                        UPDATE NOTAFISCAL
                           SET CDPARAMEMISRE = @V_C_CDPARAMEMISRE,
                               NRSEQCAPADOC = @V_NUMERORE
                         WHERE CDFILIAL = @V_NE_CDFILIAL
                           AND NRLANCTONF = @V_NE_NRLANCTONF;
                     END;
                     FETCH NEXT FROM C_NOTAFISCALE INTO @V_NE_CDFILIAL,   @V_NE_NRLANCTONF, @V_NE_CDMODDOCFISC,
                                                        @V_NE_CDTIPOOPER, @V_NE_CDFORNECED, @V_NE_CDCLIENTE;
                  END;
                  CLOSE C_NOTAFISCALE;
                  DEALLOCATE C_NOTAFISCALE;

                  FETCH NEXT FROM C_PARAMEMIS INTO @V_C_CDPARAMEMISRE, @V_C_DSOBPARAMEMISRE;
               END;
               CLOSE C_PARAMEMIS;
               DEALLOCATE C_PARAMEMIS;

               /*   Verifica notas que no possuem nenhuma regra e gera uma capa de documento (maior capa do dia) */
               SET @V_IDSEQCRIADO = 'N';

               /* Notas fiscais de entrada */
               DECLARE C_NOTAFISCALE CURSOR FOR
                  SELECT DISTINCT N.CDFILIAL, N.NRLANCTONF, N.CDMODDOCFISC, N.CDTIPOOPER, ISNULL(N.CDFORNECED,''), ISNULL(N.CDCLIENTE,'')
                    FROM NOTAFISCAL N, TIPOOPERAC T,
                         ITEMPRODUTO I LEFT JOIN ALMOXARIFADO A ON (I.CDFILIAL = A.CDFILIAL)         AND
                                                                   (I.CDALMOXARIFE = A.CDALMOXARIFE)
                                       LEFT JOIN LOCALESTOQUE L ON (I.CDFILIAL = L.CDFILIAL)         AND
                                                                   (I.CDALMOXARIFE = L.CDALMOXARIFE) AND
                                                                   (I.CDLOCALESTOQ = L.CDLOCALESTOQ)
                   WHERE N.CDFILIRECE = @P_CDFILIAL
                     AND N.IDENTRSAIDOP = 'E'
                     AND N.DTENTRSAID = @P_DATARE
                     AND (N.IDSISTEMA IN ('EST','DFE') OR N.IDSISTEMA IS NULL)
                     AND N.CDTIPOOPER NOT IN ('10', '09')
                     AND N.NRSEQCAPADOC IS NULL
                     AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                     AND N.CDTIPOOPER = T.CDTIPOOPER
                     AND ISNULL(T.IDCOMPEMISRE, 'F') = 'F'
                     AND I.CDFILIAL = N.CDFILIAL
                     AND I.NRLANCTONF = N.NRLANCTONF
                     AND ((I.CDALMOXARIFE = @P_CDALMOXARIFE) OR (@P_CDALMOXARIFE = 'T'))
                     AND ((I.CDLOCALESTOQ = @P_CDLOCALESTOQ) OR (@P_CDLOCALESTOQ = 'T'));

               OPEN C_NOTAFISCALE;
               FETCH NEXT FROM C_NOTAFISCALE INTO @V_NE_CDFILIAL,   @V_NE_NRLANCTONF, @V_NE_CDMODDOCFISC,
                                                  @V_NE_CDTIPOOPER, @V_NE_CDFORNECED, @V_NE_CDCLIENTE;
               WHILE (@@FETCH_STATUS = 0)
               BEGIN
                  IF @V_IDSEQCRIADO <> 'S' BEGIN
                     IF ((@V_IDREINICIANUMRE = 'S') AND
                         (@P_DATARE > @V_DTFECMES)   AND
                         (@V_NRMAIORRE IS NULL))    BEGIN

                        /* Verifica se j possui RE (Capa de documento) gerada para o dia */
                        SELECT @V_NUMERORE = MAX(NRNUMERNF)
                          FROM NUMERORNF
                         WHERE CDFILIAL = @P_CDFILIAL
                           AND DTNUMERNF = @P_DATARE
                           AND IDENTRSAID = 'E'
                           AND IDTIPOOPERNF = 'E';

                        IF @V_NUMERORE IS NULL BEGIN
                           UPDATE NOVOCODIGO
                              SET NRSEQUENCIAL = RIGHT(REPLICATE('0',20) + '0',20)
                            WHERE CDCONTADOR = 'RENOTAFISCAL' + @P_CDFILIAL;
                        END;
                     END;

                     SET @V_PRXNUMRE = NULL;

                     SET @V_NOVO_CODIGO = 'RENOTAFISCAL'+@P_CDFILIAL;
                     EXEC NOVO_CODIGO @V_NOVO_CODIGO, @P_SEQUENCIAL = @V_PRXNUMRE OUTPUT, @P_QTDE = 1;
                     SET @V_NUMERORE = SUBSTRING(@V_PRXNUMRE, 11, 10);

                     /* Insere novo nmero de RE */
                     INSERT INTO NUMERORNF
                       (CDFILIAL, DTNUMERNF, IDENTRSAID, IDTIPOOPERNF, NRNUMERNF)
                     VALUES
                       (@P_CDFILIAL, @P_DATARE, @P_IDENTRSAID, @P_IDENTRSAID, @V_NUMERORE);

                     UPDATE PARAMFILIAL
                        SET NRRENOTAFISC = @V_NUMERORE
                      WHERE CDFILIAL = @P_CDFILIAL;

                     SET @V_IDSEQCRIADO = 'S';
                  END;
                  UPDATE NOTAFISCAL
                     SET NRSEQCAPADOC = @V_NUMERORE
                   WHERE CDFILIAL = @V_NE_CDFILIAL
                     AND NRLANCTONF = @V_NE_NRLANCTONF;

                  FETCH NEXT FROM C_NOTAFISCALE INTO @V_NE_CDFILIAL,   @V_NE_NRLANCTONF, @V_NE_CDMODDOCFISC,
                                                     @V_NE_CDTIPOOPER, @V_NE_CDFORNECED, @V_NE_CDCLIENTE;
               END;
               CLOSE C_NOTAFISCALE;
               DEALLOCATE C_NOTAFISCALE;
            END
            ELSE BEGIN
               /* Verifica se j possui RE (Capa de documento) gerada para o dia */
               SET @V_NUMERORE = NULL;

               SELECT @V_NUMERORE = MAX(N.NRSEQCAPADOC)
                 FROM NOTAFISCAL N
                WHERE N.CDFILIRECE = @P_CDFILIAL
                  AND N.IDENTRSAIDOP = 'E'
                  AND N.DTENTRSAID = @P_DATARE
                  AND (N.IDSISTEMA IN ('EST','DFE') OR N.IDSISTEMA IS NULL)
                  AND NOT N.NRSEQCAPADOC IS NULL;

               SET @V_CDREGRA = NULL;

               SELECT @V_CDREGRA = MAX(N.CDPARAMEMISRE)
                 FROM NOTAFISCAL N
                WHERE N.CDFILIRECE = @P_CDFILIAL
                  AND N.IDENTRSAIDOP = 'E'
                  AND N.DTENTRSAID = @P_DATARE
                  AND (N.IDSISTEMA IN ('EST','DFE') OR N.IDSISTEMA IS NULL)
                  AND N.NRSEQCAPADOC = @V_NUMERORE;

               SET @V_IDSEQCRIADO = 'N';

               DECLARE C_NOTAFISCALE CURSOR FOR
                  SELECT DISTINCT N.CDFILIAL, N.NRLANCTONF, N.CDMODDOCFISC, N.CDTIPOOPER, ISNULL(N.CDFORNECED,''), ISNULL(N.CDCLIENTE,'')
                    FROM NOTAFISCAL N, TIPOOPERAC T,
                         ITEMPRODUTO I LEFT JOIN ALMOXARIFADO A ON (I.CDFILIAL = A.CDFILIAL)         AND
                                                                   (I.CDALMOXARIFE = A.CDALMOXARIFE)
                                       LEFT JOIN LOCALESTOQUE L ON (I.CDFILIAL = L.CDFILIAL)         AND
                                                                   (I.CDALMOXARIFE = L.CDALMOXARIFE) AND
                                                                   (I.CDLOCALESTOQ = L.CDLOCALESTOQ)
                   WHERE N.CDFILIRECE = @P_CDFILIAL
                     AND N.IDENTRSAIDOP = 'E'
                     AND N.DTENTRSAID = @P_DATARE
                     AND (N.IDSISTEMA IN ('EST','DFE') OR N.IDSISTEMA IS NULL)
                     AND N.CDTIPOOPER NOT IN ('10', '09')
                     AND N.NRSEQCAPADOC IS NULL
                     AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                     AND N.CDTIPOOPER = T.CDTIPOOPER
                     AND ISNULL(T.IDCOMPEMISRE, 'F') = 'F'
                     AND I.CDFILIAL = N.CDFILIAL
                     AND I.NRLANCTONF = N.NRLANCTONF
                     AND ((I.CDALMOXARIFE = @P_CDALMOXARIFE) OR (@P_CDALMOXARIFE = 'T'))
                     AND ((I.CDLOCALESTOQ = @P_CDLOCALESTOQ) OR (@P_CDLOCALESTOQ = 'T'));

               OPEN C_NOTAFISCALE;
               FETCH NEXT FROM C_NOTAFISCALE INTO @V_NE_CDFILIAL,   @V_NE_NRLANCTONF, @V_NE_CDMODDOCFISC,
                                                  @V_NE_CDTIPOOPER, @V_NE_CDFORNECED, @V_NE_CDCLIENTE;
               WHILE (@@FETCH_STATUS = 0)
               BEGIN
                  IF @V_NUMERORE IS NULL BEGIN
                     IF @V_IDSEQCRIADO <> 'S' BEGIN
                        IF ((@V_IDREINICIANUMRE = 'S') AND
                            (@P_DATARE > @V_DTFECMES)   AND
                            (@V_NRMAIORRE IS NULL))    BEGIN
                           UPDATE NOVOCODIGO
                              SET NRSEQUENCIAL = RIGHT(REPLICATE('0',20) + '0',20)
                            WHERE CDCONTADOR = 'RENOTAFISCAL' + @P_CDFILIAL;
                        END;

                        SET @V_PRXNUMRE = NULL;

                        SET @V_NOVO_CODIGO = 'RENOTAFISCAL'+@P_CDFILIAL;
                        EXEC NOVO_CODIGO @V_NOVO_CODIGO, @P_SEQUENCIAL = @V_PRXNUMRE OUTPUT, @P_QTDE = 1;
                        SET @V_NUMERORE = SUBSTRING(@V_PRXNUMRE, 11, 10);

                        /* Insere novo nmero de RE */
                        INSERT INTO NUMERORNF
                          (CDFILIAL, DTNUMERNF, IDENTRSAID, IDTIPOOPERNF, NRNUMERNF)
                        VALUES
                          (@P_CDFILIAL, @P_DATARE, @P_IDENTRSAID, @P_IDENTRSAID, @V_NUMERORE);

                        UPDATE PARAMFILIAL
                           SET NRRENOTAFISC = @V_NUMERORE
                         WHERE CDFILIAL = @P_CDFILIAL;

                        SET @V_IDSEQCRIADO = 'S';
                     END;
                  END;

                  UPDATE NOTAFISCAL
                     SET NRSEQCAPADOC = @V_NUMERORE,
                         CDPARAMEMISRE = @V_CDREGRA
                   WHERE CDFILIAL = @V_NE_CDFILIAL
                     AND NRLANCTONF = @V_NE_NRLANCTONF;

                  FETCH NEXT FROM C_NOTAFISCALE INTO @V_NE_CDFILIAL,   @V_NE_NRLANCTONF, @V_NE_CDMODDOCFISC,
                                                     @V_NE_CDTIPOOPER, @V_NE_CDFORNECED, @V_NE_CDCLIENTE;
               END;
               CLOSE C_NOTAFISCALE;
               DEALLOCATE C_NOTAFISCALE;
            END;
         END;
      END
      ELSE BEGIN
         /* Gerao de RE de Sada */

         SET @V_REGERADOE = NULL;

         SELECT @V_REGERADOE = MAX(N.CDFILIAL)
           FROM NOTAFISCAL N, TIPOOPERAC T,
                ITEMPRODUTO I LEFT JOIN ALMOXARIFADO A ON (I.CDFILIAL = A.CDFILIAL)         AND
                                                          (I.CDALMOXARIFE = A.CDALMOXARIFE)
                              LEFT JOIN LOCALESTOQUE L ON (I.CDFILIAL = L.CDFILIAL)         AND
                                                          (I.CDALMOXARIFE = L.CDALMOXARIFE) AND
                                                          (I.CDLOCALESTOQ = L.CDLOCALESTOQ)
          WHERE N.CDFILIRECE = @P_CDFILIAL
            AND N.IDENTRSAIDOP = 'E'
            AND N.DTENTRSAID = @P_DATARE
            AND (N.IDSISTEMA IN ('EST','DFE') OR N.IDSISTEMA IS NULL)
            AND N.CDTIPOOPER NOT IN ('10', '09')
            AND N.NRSEQCAPADOC IS NULL
            AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
            AND N.CDTIPOOPER = T.CDTIPOOPER
            AND ISNULL(T.IDCOMPEMISRE, 'F') = 'F'
            AND I.CDFILIAL = N.CDFILIAL
            AND I.NRLANCTONF = N.NRLANCTONF
            AND ((I.CDALMOXARIFE = @P_CDALMOXARIFE) OR (@P_CDALMOXARIFE = 'T'))
            AND ((I.CDLOCALESTOQ = @P_CDLOCALESTOQ) OR (@P_CDLOCALESTOQ = 'T'));

         IF @V_REGERADOE IS NULL BEGIN

            SET @V_REGERADO = 'N';

            /* Verifica se possui algum RE posterior a data do RE gerado */
            SELECT @V_NRREPOST = MIN(NRNUMERNF)
              FROM NUMERORNF
             WHERE CDFILIAL = @P_CDFILIAL
               AND DTNUMERNF > @P_DATARE;

            IF @V_NRREPOST IS NULL BEGIN

               /* Verifica se a data do RE  igual a data de processamento da filial,
                  caso seja, ser gerado novamente o RE de notas fiscais */
               IF @V_DTPROCESSA = @P_DATARE BEGIN
                  UPDATE NOTAFISCAL
                     SET CDPARAMEMISRE = NULL,
                         NRSEQCAPADOC = NULL
                   WHERE CDFILIEMIT = @P_CDFILIAL
                     AND IDENTRSAIDOP = @P_IDENTRSAID
                     AND DTENTRSAID = @P_DATARE
                     AND (IDSISTEMA IN ('EST','DFS','FAI') OR IDSISTEMA IS NULL);

                  DELETE FROM NUMERORNF
                   WHERE CDFILIAL = @P_CDFILIAL
                     AND DTNUMERNF = @P_DATARE
                     AND IDENTRSAID = @P_IDENTRSAID
                     AND IDTIPOOPERNF = @P_IDENTRSAID;

                  /* Retorna sequencial do dia ltimo dia gerado, caso seja cancelado o RE do dia */
                  SET @V_ANTNUMRE = NULL;
                  SELECT @V_ANTNUMRE = RIGHT(REPLICATE('0',20) + MAX(NRNUMERNF),20)
                    FROM NUMERORNF
                   WHERE CDFILIAL = @P_CDFILIAL
                     AND DTNUMERNF = (SELECT MAX(DTNUMERNF)
                                        FROM NUMERORNF
                                       WHERE CDFILIAL = @P_CDFILIAL
                                         AND DTNUMERNF <= (@P_DATARE));

                  IF @V_ANTNUMRE IS NULL BEGIN
                     SET @V_ANTNUMRE = RIGHT(REPLICATE('0',20) + '0',20);
                  END;

                  UPDATE NOVOCODIGO
                     SET NRSEQUENCIAL = @V_ANTNUMRE
                   WHERE CDCONTADOR = 'RENOTAFISCAL' + @P_CDFILIAL;
               END;
            END
            ELSE BEGIN
               SET @V_NUMERORE = NULL;

               /* Verifica se j possui RE (Capa de documento) gerada para o dia */

               SELECT @V_NUMERORE = MAX(N.NRSEQCAPADOC)
                 FROM NOTAFISCAL N
                WHERE N.CDFILIEMIT = @P_CDFILIAL
                  AND N.IDENTRSAIDOP = 'S'
                  AND N.DTENTRSAID = @P_DATARE
                  AND (N.IDSISTEMA IN ('EST','DFS','FAI') OR N.IDSISTEMA IS NULL)
                  AND N.IDSTATUS <> 'C'
                  AND NOT N.NRSEQCAPADOC IS NULL;

               IF NOT @V_NUMERORE IS NULL BEGIN
                  SET @V_CDREGRA = NULL;

                  SELECT @V_CDREGRA = MAX(N.CDPARAMEMISRE)
                    FROM NOTAFISCAL N
                   WHERE N.CDFILIEMIT = @P_CDFILIAL
                     AND N.IDENTRSAIDOP = 'S'
                     AND N.DTENTRSAID = @P_DATARE
                     AND (N.IDSISTEMA IN ('EST','DFS','FAI') OR N.IDSISTEMA IS NULL)
                     AND N.IDSTATUS <> 'C'
                     AND N.NRSEQCAPADOC = @V_NUMERORE;

                  DECLARE C_NOTAFISCALS CURSOR FOR
                     SELECT DISTINCT N.CDFILIAL, N.NRLANCTONF, N.CDMODDOCFISC, N.CDTIPOOPER, ISNULL(N.CDFORNECED,''), ISNULL(N.CDCLIENTE,'')
                       FROM NOTAFISCAL N, TIPOOPERAC T,
                            ITEMPRODUTO I LEFT JOIN ALMOXARIFADO A ON (I.CDFILIAL = A.CDFILIAL) AND
                                                                      (I.CDALMOXARIFE = A.CDALMOXARIFE)
                                          LEFT JOIN LOCALESTOQUE L ON (I.CDFILIAL = L.CDFILIAL)         AND
                                                                      (I.CDALMOXARIFE = L.CDALMOXARIFE) AND
                                                                      (I.CDLOCALESTOQ = L.CDLOCALESTOQ)
                      WHERE N.CDFILIEMIT = @P_CDFILIAL
                        AND N.IDENTRSAIDOP = 'S'
                        AND N.DTENTRSAID = @P_DATARE
                        AND (N.IDSISTEMA IN ('EST','DFS','FAI') OR N.IDSISTEMA IS NULL)
                        AND N.IDSTATUS <> 'C'
                        AND N.NRSEQCAPADOC IS NULL
                        AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                        AND N.CDTIPOOPER = T.CDTIPOOPER
                        AND ISNULL(T.IDCOMPEMISRE, 'F') = 'F'
                        AND I.CDFILIAL = N.CDFILIAL
                        AND I.NRLANCTONF = N.NRLANCTONF
                        AND ((I.CDALMOXARIFE = @P_CDALMOXARIFE) OR (@P_CDALMOXARIFE = 'T'))
                        AND ((I.CDLOCALESTOQ = @P_CDLOCALESTOQ) OR (@P_CDLOCALESTOQ = 'T'))
                        AND (NOT EXISTS (SELECT @P_CDFILIAL
                                           FROM FILIAL FL
                                          WHERE FL.CDFILIAL = N.CDFILIEMIT
                                            AND N.DTENTRSAID = @P_DATARE
                                            AND FL.CDFILIFISC <> N.CDFILIEMIT
                                            AND FL.CDFILIFISC IS NOT NULL
                                            AND N.CDTIPOOPER IN ('01','12','13')));
                  OPEN C_NOTAFISCALS;
                  FETCH NEXT FROM C_NOTAFISCALS INTO @V_NS_CDFILIAL,   @V_NS_NRLANCTONF, @V_NS_CDMODDOCFISC,
                                                     @V_NS_CDTIPOOPER, @V_NS_CDFORNECED, @V_NS_CDCLIENTE;
                  WHILE (@@FETCH_STATUS = 0)
                  BEGIN
                     UPDATE NOTAFISCAL
                        SET NRSEQCAPADOC = @V_NUMERORE,
                            CDPARAMEMISRE = @V_CDREGRA
                      WHERE CDFILIAL = @V_NS_CDFILIAL
                        AND NRLANCTONF = @V_NS_NRLANCTONF;

                     FETCH NEXT FROM C_NOTAFISCALS INTO @V_NS_CDFILIAL,   @V_NS_NRLANCTONF, @V_NS_CDMODDOCFISC,
                                                        @V_NS_CDTIPOOPER, @V_NS_CDFORNECED, @V_NS_CDCLIENTE;

                  END;
                  CLOSE C_NOTAFISCALS;
                  DEALLOCATE C_NOTAFISCALS;

               END;

               SET @V_REGERADO = 'S';
            END;

            IF @V_REGERADO = 'N' BEGIN

               /* Verifica a data de fechamento do ms de dezembro */
               SET @V_NUMRENANO = NULL;

               SELECT @V_NUMRENANO = MAX(NRNUMERNF)
                 FROM NUMERORNF
                WHERE CDFILIAL = @P_CDFILIAL
                  AND DTNUMERNF >= CONVERT(DATETIME, '01/01/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)));


               SET @V_DTFECMESANT = CONVERT(DATETIME, '01/01/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)));

               SELECT @V_DTFECMESANT = DTFECMES + 1
                 FROM DIAFECMES
                WHERE CDFILIAL = @P_CDFILIAL
                  AND DTFECMES BETWEEN CONVERT(DATETIME, '01/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)-1)) AND CONVERT(DATETIME, '31/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)-1));


               SET @V_NUMRENPER = NULL;

               SELECT @V_NUMRENPER = MAX(NRNUMERNF)
                 FROM NUMERORNF
                WHERE CDFILIAL = @P_CDFILIAL
                  AND DTNUMERNF BETWEEN @V_DTFECMESANT AND @P_DATARE;

               IF (@V_NUMRENANO IS NULL  AND
                   @V_NUMRENPER IS NULL) BEGIN
                  SET @V_DTFECMES = CONVERT(DATETIME, '31/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)-1));

                  SELECT @V_DTFECMES = MAX(DTFECMES)
                    FROM DIAFECMES
                   WHERE CDFILIAL = @P_CDFILIAL
                     AND DTFECMES BETWEEN CONVERT(DATETIME, '01/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)-1)) AND CONVERT(DATETIME, '31/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)-1));
               END
               ELSE BEGIN
                  SET @V_DTFECMES = CONVERT(DATETIME, '31/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)));

                  SELECT @V_DTFECMES = MAX(DTFECMES)
                    FROM DIAFECMES
                   WHERE CDFILIAL = @P_CDFILIAL
                     AND DTFECMES BETWEEN CONVERT(DATETIME, '01/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE))) AND CONVERT(DATETIME, '31/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)));
               END;

               /* Verifica se existe algum RE posterior a data de fechamento do ms de dezembro do ano anterior */

               SET @V_NRMAIORRE = NULL;

               SELECT @V_NRMAIORRE = MAX(NRNUMERNF)
                 FROM NUMERORNF
                WHERE CDFILIAL = @P_CDFILIAL
                  AND DTNUMERNF >= @P_DATARE
                  AND DTNUMERNF > @V_DTFECMES;

               /* Utiliza parametrizao para emisso do RE (Regras) e possui regras */
               IF (@V_IDPARAMEMISRE = 'S' AND NOT @V_CDPARAMEMISRE IS NULL) BEGIN
                  /* Regras disponveis para a filial para gerao da capa de documento (RE) */
                  DECLARE C_PARAMEMIS CURSOR FOR
                     SELECT CDPARAMEMISRE, DSOBPARAMEMISRE
                       FROM PARAMEMISSAORE
                      WHERE ISNULL(CDTIPOFATEMISRE, 'T') = @V_PCDTIPOCONT
                        AND CDEMPRESA = @V_PCDEMPRESA
                        AND ISNULL(CDINSCESTA, 'T') = @V_PCDINSCESTA
                        AND IDENTSAIDEMISRE = @P_IDENTRSAID
                        AND IDSITUQUEBRARE = 'A'
                      ORDER BY CDPRIOEMISSAORE;
                  OPEN C_PARAMEMIS;

                  FETCH NEXT FROM C_PARAMEMIS INTO @V_C_CDPARAMEMISRE, @V_C_DSOBPARAMEMISRE;
                  WHILE (@@FETCH_STATUS = 0)
                  BEGIN
                     SET @V_IDSEQCRIADO = 'N';

                     SET @V_CDMODEDOCU = '';
                     SET @V_CDTIPOOPER = '';
                     SET @V_CDFORNECED = '';
                     SET @V_CDCLIENTE  = '';

                     DECLARE C_MODEDOCU CURSOR FOR
                        SELECT CDMODDOCFISC
                          FROM MODDOCEMISSAORE
                         WHERE CDPARAMEMISRE = @V_C_CDPARAMEMISRE;
                     OPEN C_MODEDOCU;

                     FETCH NEXT FROM C_MODEDOCU INTO @V_C_CDMODDOCFISC;
                     WHILE (@@FETCH_STATUS = 0)
                     BEGIN
                        SET @V_CDMODEDOCU = @V_CDMODEDOCU + @V_C_CDMODDOCFISC + '|';

                        FETCH NEXT FROM C_MODEDOCU INTO @V_C_CDMODDOCFISC;
                     END;
                     CLOSE C_MODEDOCU;
                     DEALLOCATE C_MODEDOCU;

                     DECLARE C_TIPOOPER CURSOR FOR
                        SELECT CDTIPOOPER
                          FROM TIPOPERACEMISRE
                         WHERE CDPARAMEMISRE = @V_C_CDPARAMEMISRE
                           AND IDENTRSAIDOP = @P_IDENTRSAID;
                     OPEN C_TIPOOPER;

                     FETCH NEXT FROM C_TIPOOPER INTO @V_C_CDTIPOOPER;
                     WHILE (@@FETCH_STATUS = 0)
                     BEGIN
                        SET @V_CDTIPOOPER = @V_CDTIPOOPER + @V_C_CDTIPOOPER + '|';
                        FETCH NEXT FROM C_TIPOOPER INTO @V_C_CDTIPOOPER;
                     END;
                     CLOSE C_TIPOOPER;
                     DEALLOCATE C_TIPOOPER;

                     DECLARE C_FORNECED CURSOR FOR
                        SELECT CDFORNECED
                          FROM FORNECEMISSAORE
                         WHERE CDPARAMEMISRE = @V_C_CDPARAMEMISRE;
                     OPEN C_FORNECED;

                     FETCH NEXT FROM C_FORNECED INTO @V_C_CDFORNECED;
                     WHILE (@@FETCH_STATUS = 0)
                     BEGIN
                        SET @V_CDFORNECED = @V_CDFORNECED + @V_C_CDFORNECED + '|';
                        FETCH NEXT FROM C_FORNECED INTO @V_C_CDFORNECED;
                     END;
                     CLOSE C_FORNECED;
                     DEALLOCATE C_FORNECED;

                     DECLARE C_CLIENTE CURSOR FOR
                        SELECT CDCLIENTE
                          FROM CLIENTEMISSAORE
                         WHERE CDPARAMEMISRE = @V_C_CDPARAMEMISRE;
                     OPEN C_CLIENTE;

                     FETCH NEXT FROM C_CLIENTE INTO @V_C_CDCLIENTE;
                     WHILE (@@FETCH_STATUS = 0)
                     BEGIN
                        SET @V_CDCLIENTE = @V_CDCLIENTE + @V_C_CDCLIENTE + '|';
                        FETCH NEXT FROM C_CLIENTE INTO @V_C_CDCLIENTE;
                     END;
                     CLOSE C_CLIENTE;
                     DEALLOCATE C_CLIENTE;

                     /* Notas fiscais de sada */
                     DECLARE C_NOTAFISCALS CURSOR FOR
                        SELECT DISTINCT N.CDFILIAL, N.NRLANCTONF, N.CDMODDOCFISC, N.CDTIPOOPER, ISNULL(N.CDFORNECED,''), ISNULL(N.CDCLIENTE,'')
                          FROM NOTAFISCAL N, TIPOOPERAC T,
                               ITEMPRODUTO I LEFT JOIN ALMOXARIFADO A ON (I.CDFILIAL = A.CDFILIAL) AND
                                                                         (I.CDALMOXARIFE = A.CDALMOXARIFE)
                                             LEFT JOIN LOCALESTOQUE L ON (I.CDFILIAL = L.CDFILIAL)         AND
                                                                         (I.CDALMOXARIFE = L.CDALMOXARIFE) AND
                                                                         (I.CDLOCALESTOQ = L.CDLOCALESTOQ)
                         WHERE N.CDFILIEMIT = @P_CDFILIAL
                           AND N.IDENTRSAIDOP = 'S'
                           AND N.DTENTRSAID = @P_DATARE
                           AND (N.IDSISTEMA IN ('EST','DFS','FAI') OR N.IDSISTEMA IS NULL)
                           AND N.IDSTATUS <> 'C'
                           AND N.NRSEQCAPADOC IS NULL
                           AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                           AND N.CDTIPOOPER = T.CDTIPOOPER
                           AND ISNULL(T.IDCOMPEMISRE, 'F') = 'F'
                           AND I.CDFILIAL = N.CDFILIAL
                           AND I.NRLANCTONF = N.NRLANCTONF
                           AND ((I.CDALMOXARIFE = @P_CDALMOXARIFE) OR (@P_CDALMOXARIFE = 'T'))
                           AND ((I.CDLOCALESTOQ = @P_CDLOCALESTOQ) OR (@P_CDLOCALESTOQ = 'T'))
                           AND (NOT EXISTS (SELECT @P_CDFILIAL
                                              FROM FILIAL FL
                                             WHERE FL.CDFILIAL = N.CDFILIEMIT
                                               AND N.DTENTRSAID = @P_DATARE
                                               AND FL.CDFILIFISC <> N.CDFILIEMIT
                                               AND FL.CDFILIFISC IS NOT NULL
                                               AND N.CDTIPOOPER IN ('01','12','13')));
                     OPEN C_NOTAFISCALS;
                     FETCH NEXT FROM C_NOTAFISCALS INTO @V_NS_CDFILIAL,   @V_NS_NRLANCTONF, @V_NS_CDMODDOCFISC,
                                                        @V_NS_CDTIPOOPER, @V_NS_CDFORNECED, @V_NS_CDCLIENTE;
                     WHILE (@@FETCH_STATUS = 0)
                     BEGIN
                        SET @V_IDCAPACORRES = 'S';

                        IF (NOT @V_CDMODEDOCU = '' AND @V_IDCAPACORRES = 'S') BEGIN
                           IF CHARINDEX( @V_NS_CDMODDOCFISC, @V_CDMODEDOCU) = 0 BEGIN
                              SET @V_IDCAPACORRES = 'N';
                           END;
                        END;

                        IF (NOT @V_CDTIPOOPER = '' AND @V_IDCAPACORRES = 'S') BEGIN
                           IF CHARINDEX( @V_NS_CDTIPOOPER, @V_CDTIPOOPER) = 0 BEGIN
                              SET @V_IDCAPACORRES = 'N';
                           END;
                        END;

                        IF (NOT @V_CDFORNECED = '' AND @V_IDCAPACORRES = 'S') OR
                           (NOT @V_CDCLIENTE = '' AND @V_IDCAPACORRES = 'S')  BEGIN
                           IF (CHARINDEX( @V_NS_CDFORNECED, @V_CDFORNECED) = 0) AND
                              (CHARINDEX( @V_NS_CDCLIENTE, @V_CDCLIENTE) = 0)   BEGIN
                              SET @V_IDCAPACORRES = 'N';
                           END;
                        END;

                        /* Se atende as regras de capa de documento conforme parametrizao (regras) */
                        IF @V_IDCAPACORRES = 'S' BEGIN
                           /* Verifica se j possui RE (Capa de documento) gerada para o dia conforme regras */
                           SET @V_NUMERORE = NULL;

                           SELECT @V_NUMERORE = MAX(N.NRSEQCAPADOC)
                             FROM NOTAFISCAL N
                            WHERE N.CDFILIEMIT = @P_CDFILIAL
                              AND N.IDENTRSAIDOP = 'S'
                              AND N.DTENTRSAID = @P_DATARE
                              AND (N.IDSISTEMA IN ('EST','DFS','FAI') OR N.IDSISTEMA IS NULL)
                              AND N.IDSTATUS <> 'C'
                              AND NOT N.NRSEQCAPADOC IS NULL
                              AND N.CDPARAMEMISRE = @V_C_CDPARAMEMISRE;

                           IF @V_NUMERORE IS NULL BEGIN
                              IF @V_IDSEQCRIADO <> 'S' BEGIN
                                 IF ((@V_IDREINICIANUMRE = 'S') AND
                                     (@P_DATARE > @V_DTFECMES)   AND
                                     (@V_NRMAIORRE IS NULL))    BEGIN

                                    /* Verifica se j possui RE (Capa de documento) gerada para o dia independente de regras */
                                    SET @V_NUMERORED = NULL;

                                    SELECT @V_NUMERORED = MAX(N.NRSEQCAPADOC)
                                      FROM NOTAFISCAL N
                                     WHERE N.CDFILIEMIT = @P_CDFILIAL
                                       AND N.IDENTRSAIDOP = 'S'
                                       AND N.DTENTRSAID = @P_DATARE
                                       AND (N.IDSISTEMA IN ('EST','DFS','FAI') OR N.IDSISTEMA IS NULL)
                                       AND N.IDSTATUS <> 'C'
                                       AND NOT N.NRSEQCAPADOC IS NULL;

                                    IF @V_NUMERORED IS NULL BEGIN
                                       UPDATE NOVOCODIGO
                                          SET NRSEQUENCIAL = RIGHT(REPLICATE('0',20) + '0',20)
                                        WHERE CDCONTADOR = 'RENOTAFISCAL' + @P_CDFILIAL;
                                    END;
                                 END;

                                 SET @V_PRXNUMRE = NULL;

                                 SET @V_NOVO_CODIGO = 'RENOTAFISCAL'+@P_CDFILIAL;
                                 EXEC NOVO_CODIGO @V_NOVO_CODIGO, @P_SEQUENCIAL = @V_PRXNUMRE OUTPUT, @P_QTDE = 1;
                                 SET @V_NUMERORE = SUBSTRING(@V_PRXNUMRE, 11, 10);

                                 /* Insere novo nmero de RE */
                                 INSERT INTO NUMERORNF
                                   (CDFILIAL, DTNUMERNF, IDENTRSAID, IDTIPOOPERNF, NRNUMERNF)
                                 VALUES
                                   (@P_CDFILIAL, @P_DATARE, @P_IDENTRSAID, @P_IDENTRSAID, @V_NUMERORE);

                                 UPDATE PARAMFILIAL
                                    SET NRRENOTAFISC = @V_NUMERORE
                                  WHERE CDFILIAL = @P_CDFILIAL;

                                 SET @V_IDSEQCRIADO = 'S';
                              END;
                           END;
                           UPDATE NOTAFISCAL
                              SET CDPARAMEMISRE = @V_C_CDPARAMEMISRE,
                                  NRSEQCAPADOC = @V_NUMERORE
                            WHERE CDFILIAL =  @V_NS_CDFILIAL
                              AND NRLANCTONF =  @V_NS_NRLANCTONF;
                        END;
                        FETCH NEXT FROM C_NOTAFISCALS INTO @V_NS_CDFILIAL,   @V_NS_NRLANCTONF, @V_NS_CDMODDOCFISC,
                                                           @V_NS_CDTIPOOPER, @V_NS_CDFORNECED, @V_NS_CDCLIENTE;
                     END;
                     CLOSE C_NOTAFISCALS;
                     DEALLOCATE C_NOTAFISCALS;

                     FETCH NEXT FROM C_PARAMEMIS INTO @V_C_CDPARAMEMISRE, @V_C_DSOBPARAMEMISRE;
                  END;
                  CLOSE C_PARAMEMIS;
                  DEALLOCATE C_PARAMEMIS;

                  /*   Verifica notas que no possuem nenhuma regra e gera uma capa de documento (maior capa do dia) */
                  SET @V_IDSEQCRIADO = 'N';

                  /* Notas fiscais de sada */
                  DECLARE C_NOTAFISCALS CURSOR FOR
                     SELECT DISTINCT N.CDFILIAL, N.NRLANCTONF, N.CDMODDOCFISC, N.CDTIPOOPER, ISNULL(N.CDFORNECED,''), ISNULL(N.CDCLIENTE,'')
                       FROM NOTAFISCAL N, TIPOOPERAC T,
                            ITEMPRODUTO I LEFT JOIN ALMOXARIFADO A ON (I.CDFILIAL = A.CDFILIAL) AND
                                                                      (I.CDALMOXARIFE = A.CDALMOXARIFE)
                                          LEFT JOIN LOCALESTOQUE L ON (I.CDFILIAL = L.CDFILIAL)         AND
                                                                      (I.CDALMOXARIFE = L.CDALMOXARIFE) AND
                                                                      (I.CDLOCALESTOQ = L.CDLOCALESTOQ)
                      WHERE N.CDFILIEMIT = @P_CDFILIAL
                        AND N.IDENTRSAIDOP = 'S'
                        AND N.DTENTRSAID = @P_DATARE
                        AND (N.IDSISTEMA IN ('EST','DFS','FAI') OR N.IDSISTEMA IS NULL)
                        AND N.IDSTATUS <> 'C'
                        AND N.NRSEQCAPADOC IS NULL
                        AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                        AND N.CDTIPOOPER = T.CDTIPOOPER
                        AND ISNULL(T.IDCOMPEMISRE, 'F') = 'F'
                        AND I.CDFILIAL = N.CDFILIAL
                        AND I.NRLANCTONF = N.NRLANCTONF
                        AND ((I.CDALMOXARIFE = @P_CDALMOXARIFE) OR (@P_CDALMOXARIFE = 'T'))
                        AND ((I.CDLOCALESTOQ = @P_CDLOCALESTOQ) OR (@P_CDLOCALESTOQ = 'T'))
                        AND (NOT EXISTS (SELECT @P_CDFILIAL
                                           FROM FILIAL FL
                                          WHERE FL.CDFILIAL = N.CDFILIEMIT
                                            AND N.DTENTRSAID = @P_DATARE
                                            AND FL.CDFILIFISC <> N.CDFILIEMIT
                                            AND FL.CDFILIFISC IS NOT NULL
                                            AND N.CDTIPOOPER IN ('01','12','13')));
                  OPEN C_NOTAFISCALS;
                  FETCH NEXT FROM C_NOTAFISCALS INTO @V_NS_CDFILIAL,   @V_NS_NRLANCTONF, @V_NS_CDMODDOCFISC,
                                                     @V_NS_CDTIPOOPER, @V_NS_CDFORNECED, @V_NS_CDCLIENTE;

                  WHILE (@@FETCH_STATUS = 0)
                  BEGIN
                     IF @V_IDSEQCRIADO <> 'S' BEGIN
                        IF ((@V_IDREINICIANUMRE = 'S') AND
                            (@P_DATARE > @V_DTFECMES)   AND
                            (@V_NRMAIORRE IS NULL))    BEGIN

                           /* Verifica se j possui RE (Capa de documento) gerada para o dia */
                           SELECT @V_NUMERORE = MAX(NRNUMERNF)
                             FROM NUMERORNF
                            WHERE CDFILIAL = @P_CDFILIAL
                              AND DTNUMERNF = @P_DATARE
                              AND IDENTRSAID = 'S'
                              AND IDTIPOOPERNF = 'S';

                           IF @V_NUMERORE IS NULL BEGIN
                              UPDATE NOVOCODIGO
                                 SET NRSEQUENCIAL = RIGHT(REPLICATE('0',20) + '0',20)
                               WHERE CDCONTADOR = 'RENOTAFISCAL' + @P_CDFILIAL;
                           END;    
                        END;

                        SET @V_PRXNUMRE = NULL;

                        SET @V_NOVO_CODIGO = 'RENOTAFISCAL'+@P_CDFILIAL;
                        EXEC NOVO_CODIGO @V_NOVO_CODIGO, @P_SEQUENCIAL = @V_PRXNUMRE OUTPUT, @P_QTDE = 1;
                        SET @V_NUMERORE = SUBSTRING(@V_PRXNUMRE, 11, 10);

                        /* Insere novo nmero de RE */
                        INSERT INTO NUMERORNF
                          (CDFILIAL, DTNUMERNF, IDENTRSAID, IDTIPOOPERNF, NRNUMERNF)
                        VALUES
                          (@P_CDFILIAL, @P_DATARE, @P_IDENTRSAID, @P_IDENTRSAID, @V_NUMERORE);

                        UPDATE PARAMFILIAL
                           SET NRRENOTAFISC = @V_NUMERORE
                         WHERE CDFILIAL = @P_CDFILIAL;

                        SET @V_IDSEQCRIADO = 'S';
                     END;
                     UPDATE NOTAFISCAL
                        SET NRSEQCAPADOC = @V_NUMERORE
                      WHERE CDFILIAL = @V_NS_CDFILIAL
                        AND NRLANCTONF = @V_NS_NRLANCTONF;

                     FETCH NEXT FROM C_NOTAFISCALS INTO @V_NS_CDFILIAL,   @V_NS_NRLANCTONF, @V_NS_CDMODDOCFISC,
                                                        @V_NS_CDTIPOOPER, @V_NS_CDFORNECED, @V_NS_CDCLIENTE;
                  END;
                  CLOSE C_NOTAFISCALS;
                  DEALLOCATE C_NOTAFISCALS;
               END
               ELSE BEGIN
                  /* Verifica se j possui RE (Capa de documento) gerada para o dia */
                  SET @V_NUMERORE = NULL;

                  SELECT @V_NUMERORE = MAX(N.NRSEQCAPADOC)
                    FROM NOTAFISCAL N
                   WHERE CDFILIEMIT = @P_CDFILIAL
                     AND IDENTRSAIDOP = @P_IDENTRSAID
                     AND DTENTRSAID = @P_DATARE
                     AND (IDSISTEMA IN ('EST','DFS','FAI') OR IDSISTEMA IS NULL)
                     AND NOT N.NRSEQCAPADOC IS NULL;

                  SET @V_CDREGRA = NULL;

                  SELECT @V_CDREGRA = MAX(N.CDPARAMEMISRE)
                    FROM NOTAFISCAL N
                   WHERE CDFILIEMIT = @P_CDFILIAL
                     AND IDENTRSAIDOP = @P_IDENTRSAID
                     AND DTENTRSAID = @P_DATARE
                     AND (IDSISTEMA IN ('EST','DFS','FAI') OR IDSISTEMA IS NULL)
                     AND N.NRSEQCAPADOC = @V_NUMERORE;

                  SET @V_IDSEQCRIADO = 'N';

                  DECLARE C_NOTAFISCALS CURSOR FOR
                     SELECT DISTINCT N.CDFILIAL, N.NRLANCTONF, N.CDMODDOCFISC, N.CDTIPOOPER, ISNULL(N.CDFORNECED,''), ISNULL(N.CDCLIENTE,'')
                       FROM NOTAFISCAL N, TIPOOPERAC T,
                            ITEMPRODUTO I LEFT JOIN ALMOXARIFADO A ON (I.CDFILIAL = A.CDFILIAL) AND
                                                                      (I.CDALMOXARIFE = A.CDALMOXARIFE)
                                          LEFT JOIN LOCALESTOQUE L ON (I.CDFILIAL = L.CDFILIAL)         AND
                                                                      (I.CDALMOXARIFE = L.CDALMOXARIFE) AND
                                                                      (I.CDLOCALESTOQ = L.CDLOCALESTOQ)
                      WHERE N.CDFILIEMIT = @P_CDFILIAL
                        AND N.IDENTRSAIDOP = 'S'
                        AND N.DTENTRSAID = @P_DATARE
                        AND (N.IDSISTEMA IN ('EST','DFS','FAI') OR N.IDSISTEMA IS NULL)
                        AND N.IDSTATUS <> 'C'
                        AND N.NRSEQCAPADOC IS NULL
                        AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                        AND N.CDTIPOOPER = T.CDTIPOOPER
                        AND ISNULL(T.IDCOMPEMISRE, 'F') = 'F'
                        AND I.CDFILIAL = N.CDFILIAL
                        AND I.NRLANCTONF = N.NRLANCTONF
                        AND ((I.CDALMOXARIFE = @P_CDALMOXARIFE) OR (@P_CDALMOXARIFE = 'T'))
                        AND ((I.CDLOCALESTOQ = @P_CDLOCALESTOQ) OR (@P_CDLOCALESTOQ = 'T'))
                        AND (NOT EXISTS (SELECT @P_CDFILIAL
                                           FROM FILIAL FL
                                          WHERE FL.CDFILIAL = N.CDFILIEMIT
                                            AND N.DTENTRSAID = @P_DATARE
                                            AND FL.CDFILIFISC <> N.CDFILIEMIT
                                            AND FL.CDFILIFISC IS NOT NULL
                                            AND N.CDTIPOOPER IN ('01','12','13')));
                  OPEN C_NOTAFISCALS;
                  FETCH NEXT FROM C_NOTAFISCALS INTO @V_NS_CDFILIAL,   @V_NS_NRLANCTONF, @V_NS_CDMODDOCFISC,
                                                     @V_NS_CDTIPOOPER, @V_NS_CDFORNECED, @V_NS_CDCLIENTE;
                  WHILE (@@FETCH_STATUS = 0)
                  BEGIN
                     IF @V_NUMERORE IS NULL BEGIN
                        IF @V_IDSEQCRIADO <> 'S' BEGIN
                           IF ((@V_IDREINICIANUMRE = 'S') AND
                               (@P_DATARE > @V_DTFECMES)   AND
                               (@V_NRMAIORRE IS NULL))    BEGIN
                              UPDATE NOVOCODIGO
                                 SET NRSEQUENCIAL = RIGHT(REPLICATE('0',20) + '0',20)
                               WHERE CDCONTADOR = 'RENOTAFISCAL' + @P_CDFILIAL;
                           END;

                           SET @V_PRXNUMRE = NULL;

                           SET @V_NOVO_CODIGO = 'RENOTAFISCAL'+@P_CDFILIAL;
                           EXEC NOVO_CODIGO @V_NOVO_CODIGO, @P_SEQUENCIAL = @V_PRXNUMRE OUTPUT, @P_QTDE = 1;
                           SET @V_NUMERORE = SUBSTRING(@V_PRXNUMRE, 11, 10);

                           /* Insere novo nmero de RE */
                           INSERT INTO NUMERORNF
                             (CDFILIAL, DTNUMERNF, IDENTRSAID, IDTIPOOPERNF, NRNUMERNF)
                           VALUES
                             (@P_CDFILIAL, @P_DATARE, @P_IDENTRSAID, @P_IDENTRSAID, @V_NUMERORE);

                           UPDATE PARAMFILIAL
                              SET NRRENOTAFISC = @V_NUMERORE
                            WHERE CDFILIAL = @P_CDFILIAL;

                           SET @V_IDSEQCRIADO = 'S';
                        END;
                     END;

                     UPDATE NOTAFISCAL
                        SET NRSEQCAPADOC = @V_NUMERORE,
                            CDPARAMEMISRE = @V_CDREGRA
                      WHERE CDFILIAL =  @V_NS_CDFILIAL
                        AND NRLANCTONF =  @V_NS_NRLANCTONF;

                     FETCH NEXT FROM C_NOTAFISCALS INTO @V_NS_CDFILIAL,   @V_NS_NRLANCTONF, @V_NS_CDMODDOCFISC,
                                                        @V_NS_CDTIPOOPER, @V_NS_CDFORNECED, @V_NS_CDCLIENTE;
                  END;
                  CLOSE C_NOTAFISCALS;
                  DEALLOCATE C_NOTAFISCALS;
               END;
            END;
         END
         ELSE BEGIN
            SET @P_IDERRO = 'S';
         END;
      END;
   END
   ELSE BEGIN
      /* Gerao do RE Centralizadora */

      SET @V_REGERADO = 'N';

      /* Verifica se possui algum RE posterior a data do RE gerado */
      SELECT @V_NRREPOST = MIN(NRNUMERNF)
        FROM NUMERORNF
       WHERE CDFILIAL = @P_CDFILIAL
         AND DTNUMERNF > @P_DATARE;

      IF @V_NRREPOST IS NULL BEGIN

         /* Verifica se a data do RE  igual a data de processamento da filial,
            caso seja, ser gerado novamente o RE de notas fiscais */
         IF @V_DTPROCESSA = @P_DATARE BEGIN
            UPDATE NOTAFISCAL
               SET CDPARAMEMISRE = NULL,
                   NRSEQCAPADOC = NULL
             WHERE IDENTRSAIDOP = 'S'
               AND IDSTATUS IN ('A','E')
               AND DTAPROVACAO = @P_DATARE
               AND (IDSISTEMA IN ('EST','DFS','FAI','PDV') OR IDSISTEMA IS NULL)
               AND (EXISTS (SELECT CDFILIAL
                             FROM FILIAL FL
                            WHERE ISNULL(CDFILIFISC,FL.CDFILIAL) = @P_CDFILIAL
                              AND NOTAFISCAL.CDFILIEMIT = FL.CDFILIAL));
            DELETE FROM NUMERORNF
             WHERE CDFILIAL = @P_CDFILIAL
               AND DTNUMERNF = @P_DATARE
               AND IDENTRSAID = @P_IDENTRSAID
               AND IDTIPOOPERNF = @P_IDENTRSAID;

            /* Retorna sequencial do dia ltimo dia gerado, caso seja cancelado o RE do dia */
            SET @V_ANTNUMRE = NULL;

            SELECT @V_ANTNUMRE = RIGHT(REPLICATE('0',20) + MAX(NRNUMERNF),20)
              FROM NUMERORNF
             WHERE CDFILIAL = @P_CDFILIAL
               AND DTNUMERNF = (SELECT MAX(DTNUMERNF)
                                  FROM NUMERORNF
                                 WHERE CDFILIAL = @P_CDFILIAL
                                   AND DTNUMERNF <= (@P_DATARE - 1));

            IF @V_ANTNUMRE IS NULL BEGIN
               SET @V_ANTNUMRE = RIGHT(REPLICATE('0',20) + '0',20);
            END;

            UPDATE NOVOCODIGO
               SET NRSEQUENCIAL = @V_ANTNUMRE
             WHERE CDCONTADOR = 'RENOTAFISCAL' + @P_CDFILIAL;
         END;
      END
      ELSE BEGIN
         SET @V_NUMERORE = NULL;

         /* Verifica se j possui RE (Capa de documento) gerada para o dia */
         SELECT @V_NUMERORE = MAX(N.NRSEQCAPADOC)
           FROM NOTAFISCAL N, TIPOOPERAC T
          WHERE N.IDENTRSAIDOP = 'S'
            AND IDSTATUS IN ('A','E')
            AND N.DTAPROVACAO   = @P_DATARE
            AND ((N.IDSISTEMA IN ('EST','DFS','FAI','PDV') OR N.IDSISTEMA IS NULL))
            AND NOT N.NRSEQCAPADOC IS NULL
            AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
            AND N.CDTIPOOPER = T.CDTIPOOPER
            AND T.IDCOMPEMISRE = 'C'
            AND (EXISTS (SELECT CDFILIAL
                           FROM FILIAL FL
                          WHERE ISNULL(CDFILIFISC,FL.CDFILIAL) = @P_CDFILIAL
                            AND N.CDFILIEMIT = FL.CDFILIAL));

         IF NOT @V_NUMERORE IS NULL BEGIN
            SET @V_CDREGRA = NULL;

            SELECT @V_CDREGRA = MAX(N.CDPARAMEMISRE)
              FROM NOTAFISCAL N, TIPOOPERAC T
             WHERE N.IDENTRSAIDOP = 'S'
               AND IDSTATUS IN ('A','E')
               AND N.DTAPROVACAO   = @P_DATARE
               AND ((N.IDSISTEMA IN ('EST','DFS','FAI','PDV') OR N.IDSISTEMA IS NULL))
               AND N.NRSEQCAPADOC = @V_NUMERORE
               AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
               AND N.CDTIPOOPER = T.CDTIPOOPER
               AND T.IDCOMPEMISRE = 'C'
               AND (EXISTS (SELECT CDFILIAL
                                 FROM FILIAL FL
                                WHERE ISNULL(CDFILIFISC,FL.CDFILIAL) = @P_CDFILIAL
                                  AND N.CDFILIEMIT = FL.CDFILIAL));

            DECLARE C_NOTAFISCALSC CURSOR FOR
               SELECT DISTINCT N.CDFILIAL, N.NRLANCTONF, N.CDMODDOCFISC, N.CDTIPOOPER, ISNULL(N.CDFORNECED,''), ISNULL(N.CDCLIENTE,'')
                 FROM FILIAL F, FILIAL I, TIPOOPERAC T,
                      NOTAFISCAL N LEFT JOIN ALMOXARIFADO A ON (N.CDALMOXARIFE = A.CDALMOXARIFE) AND
                                                                (N.CDFILIAL = A.CDFILIAL)
                                   LEFT JOIN LOCALESTOQUE L ON (N.CDALMOXARIFE = L.CDALMOXARIFE) AND
                                                               (N.CDFILIAL = L.CDFILIAL)         AND
                                                               (N.CDLOCALESTOQ = L.CDLOCALESTOQ)
                WHERE N.IDENTRSAIDOP = 'S'
                  AND IDSTATUS IN ('A','E')
                  AND N.DTAPROVACAO   = @P_DATARE
                  AND N.NRSEQCAPADOC IS NULL
                  AND ((N.IDSISTEMA IN ('EST','DFS','FAI','PDV') OR N.IDSISTEMA IS NULL))
                  AND ((N.CDALMOXARIFE = ISNULL(@P_CDALMOXARIFE, N.CDALMOXARIFE)) OR @P_CDALMOXARIFE = 'T')
                  AND ((N.CDLOCALESTOQ = ISNULL(@P_CDLOCALESTOQ, N.CDLOCALESTOQ)) OR @P_CDLOCALESTOQ = 'T')
                  AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                  AND N.CDTIPOOPER = T.CDTIPOOPER
                  AND T.IDCOMPEMISRE = 'C'
                  AND N.CDFILIEMIT = F.CDFILIAL
                  AND @P_CDFILIAL = I.CDFILIAL
                  AND (EXISTS (SELECT CDFILIAL
                                 FROM FILIAL FL
                                WHERE ISNULL(CDFILIFISC,FL.CDFILIAL) = @P_CDFILIAL
                                  AND N.CDFILIEMIT = FL.CDFILIAL));
            OPEN C_NOTAFISCALSC;
            FETCH NEXT FROM C_NOTAFISCALSC INTO @V_NSC_CDFILIAL,   @V_NSC_NRLANCTONF, @V_NSC_CDMODDOCFISC,
                                                @V_NSC_CDTIPOOPER, @V_NSC_CDFORNECED, @V_NSC_CDCLIENTE;
            WHILE (@@FETCH_STATUS = 0)
            BEGIN

               UPDATE NOTAFISCAL
                  SET NRSEQCAPADOC = @V_NUMERORE,
                      CDPARAMEMISRE = @V_CDREGRA
                WHERE CDFILIAL = @V_NSC_CDFILIAL
                  AND NRLANCTONF = @V_NSC_NRLANCTONF;

               FETCH NEXT FROM C_NOTAFISCALSC INTO @V_NSC_CDFILIAL,   @V_NSC_NRLANCTONF, @V_NSC_CDMODDOCFISC,
                                                   @V_NSC_CDTIPOOPER, @V_NSC_CDFORNECED, @V_NSC_CDCLIENTE;
            END;

            CLOSE C_NOTAFISCALSC;
            DEALLOCATE C_NOTAFISCALSC;
         END;

         SET @V_REGERADO = 'S';
      END;

      IF @V_REGERADO = 'N' BEGIN

         /* Verifica a data de fechamento do ms de dezembro */
         SET @V_NUMRENANO = NULL;

         SELECT @V_NUMRENANO = MAX(NRNUMERNF)
           FROM NUMERORNF
          WHERE CDFILIAL = @P_CDFILIAL
            AND DTNUMERNF >= CONVERT(DATETIME, '01/01/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)));


         SET @V_DTFECMESANT = CONVERT(DATETIME, '01/01/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)));

         SELECT @V_DTFECMESANT = DTFECMES + 1
           FROM DIAFECMES
          WHERE CDFILIAL = @P_CDFILIAL
            AND DTFECMES BETWEEN CONVERT(DATETIME, '01/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)-1)) AND CONVERT(DATETIME, '31/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)-1));


         SET @V_NUMRENPER = NULL;

         SELECT @V_NUMRENPER = MAX(NRNUMERNF)
           FROM NUMERORNF
          WHERE CDFILIAL = @P_CDFILIAL
            AND DTNUMERNF BETWEEN @V_DTFECMESANT AND @P_DATARE;

         IF (@V_NUMRENANO IS NULL  AND
             @V_NUMRENPER IS NULL) BEGIN
            SET @V_DTFECMES = CONVERT(DATETIME, '31/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)-1));

            SELECT @V_DTFECMES = MAX(DTFECMES)
              FROM DIAFECMES
             WHERE CDFILIAL = @P_CDFILIAL
               AND DTFECMES BETWEEN CONVERT(DATETIME, '01/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)-1)) AND CONVERT(DATETIME, '31/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)-1));
         END
         ELSE BEGIN
            SET @V_DTFECMES = CONVERT(DATETIME, '31/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)));

            SELECT @V_DTFECMES = MAX(DTFECMES)
              FROM DIAFECMES
             WHERE CDFILIAL = @P_CDFILIAL
               AND DTFECMES BETWEEN CONVERT(DATETIME, '01/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE))) AND CONVERT(DATETIME, '31/12/'+ CONVERT(VARCHAR, DATEPART(YYYY,@P_DATARE)));
         END;

         /* Verifica se existe algum RE posterior a data de fechamento do ms de dezembro do ano anterior */
         SET @V_NRMAIORRE = NULL;

         SELECT @V_NRMAIORRE = MAX(NRNUMERNF)
           FROM NUMERORNF
          WHERE CDFILIAL = @P_CDFILIAL
            AND DTNUMERNF >= @P_DATARE
            AND DTNUMERNF > @V_DTFECMES;

         /* Utiliza parametrizao para emisso do RE (Regras) e possui regras */
         IF (@V_IDPARAMEMISRE = 'S' AND NOT @V_CDPARAMEMISRE IS NULL) BEGIN
            /* Regras disponveis para a filial para gerao da capa de documento (RE) */
            DECLARE C_PARAMEMIS CURSOR FOR
               SELECT CDPARAMEMISRE, DSOBPARAMEMISRE
                 FROM PARAMEMISSAORE
                WHERE ISNULL(CDTIPOFATEMISRE, 'T') = @V_PCDTIPOCONT
                  AND CDEMPRESA = @V_PCDEMPRESA
                  AND ISNULL(CDINSCESTA, 'T') = @V_PCDINSCESTA
                  AND IDENTSAIDEMISRE = @P_IDENTRSAID
                  AND IDSITUQUEBRARE = 'A'
                ORDER BY CDPRIOEMISSAORE;
            OPEN C_PARAMEMIS;

            FETCH NEXT FROM C_PARAMEMIS INTO @V_C_CDPARAMEMISRE, @V_C_DSOBPARAMEMISRE;
            WHILE (@@FETCH_STATUS = 0)
            BEGIN
               SET @V_IDSEQCRIADO = 'N';

               SET @V_CDMODEDOCU = '';
               SET @V_CDTIPOOPER = '';
               SET @V_CDFORNECED = '';
               SET @V_CDCLIENTE  = '';

               DECLARE C_MODEDOCU CURSOR FOR
                  SELECT CDMODDOCFISC
                    FROM MODDOCEMISSAORE
                   WHERE CDPARAMEMISRE = @V_C_CDPARAMEMISRE;
               OPEN C_MODEDOCU;

               FETCH NEXT FROM C_MODEDOCU INTO @V_C_CDMODDOCFISC;
               WHILE (@@FETCH_STATUS = 0)
               BEGIN
                  SET @V_CDMODEDOCU = @V_CDMODEDOCU + @V_C_CDMODDOCFISC + '|';

                  FETCH NEXT FROM C_MODEDOCU INTO @V_C_CDMODDOCFISC;
               END;
               CLOSE C_MODEDOCU;
               DEALLOCATE C_MODEDOCU;

               DECLARE C_TIPOOPER CURSOR FOR
                  SELECT CDTIPOOPER
                    FROM TIPOPERACEMISRE
                   WHERE CDPARAMEMISRE = @V_C_CDPARAMEMISRE
                     AND IDENTRSAIDOP = @P_IDENTRSAID;
               OPEN C_TIPOOPER;

               FETCH NEXT FROM C_TIPOOPER INTO @V_C_CDTIPOOPER;
               WHILE (@@FETCH_STATUS = 0)
               BEGIN
                  SET @V_CDTIPOOPER = @V_CDTIPOOPER + @V_C_CDTIPOOPER + '|';
                  FETCH NEXT FROM C_TIPOOPER INTO @V_C_CDTIPOOPER;
               END;
               CLOSE C_TIPOOPER;
               DEALLOCATE C_TIPOOPER;

               DECLARE C_FORNECED CURSOR FOR
                  SELECT CDFORNECED
                    FROM FORNECEMISSAORE
                   WHERE CDPARAMEMISRE = @V_C_CDPARAMEMISRE;
               OPEN C_FORNECED;

               FETCH NEXT FROM C_FORNECED INTO @V_C_CDFORNECED;
               WHILE (@@FETCH_STATUS = 0)
               BEGIN
                  SET @V_CDFORNECED = @V_CDFORNECED + @V_C_CDFORNECED + '|';
                  FETCH NEXT FROM C_FORNECED INTO @V_C_CDFORNECED;
               END;
               CLOSE C_FORNECED;
               DEALLOCATE C_FORNECED;

               DECLARE C_CLIENTE CURSOR FOR
                  SELECT CDCLIENTE
                    FROM CLIENTEMISSAORE
                   WHERE CDPARAMEMISRE = @V_C_CDPARAMEMISRE;
               OPEN C_CLIENTE;

               FETCH NEXT FROM C_CLIENTE INTO @V_C_CDCLIENTE;
               WHILE (@@FETCH_STATUS = 0)
               BEGIN
                  SET @V_CDCLIENTE = @V_CDCLIENTE + @V_C_CDCLIENTE + '|';
                  FETCH NEXT FROM C_CLIENTE INTO @V_C_CDCLIENTE;
               END;
               CLOSE C_CLIENTE;
               DEALLOCATE C_CLIENTE;

               /* Notas fiscais de sada - RE Centralizadora */
               DECLARE C_NOTAFISCALSC CURSOR FOR
                  SELECT DISTINCT N.CDFILIAL, N.NRLANCTONF, N.CDMODDOCFISC, N.CDTIPOOPER, ISNULL(N.CDFORNECED,''), ISNULL(N.CDCLIENTE,'')
                    FROM FILIAL F, FILIAL I, TIPOOPERAC T,
                         NOTAFISCAL N LEFT JOIN ALMOXARIFADO A ON (N.CDALMOXARIFE = A.CDALMOXARIFE) AND
                                                                   (N.CDFILIAL = A.CDFILIAL)
                                      LEFT JOIN LOCALESTOQUE L ON (N.CDALMOXARIFE = L.CDALMOXARIFE) AND
                                                                  (N.CDFILIAL = L.CDFILIAL)         AND
                                                                  (N.CDLOCALESTOQ = L.CDLOCALESTOQ)
                   WHERE N.IDENTRSAIDOP = 'S'
                     AND IDSTATUS IN ('A','E')
                     AND N.DTAPROVACAO   = @P_DATARE
                     AND N.NRSEQCAPADOC IS NULL
                     AND ((N.IDSISTEMA IN ('EST','DFS','FAI','PDV') OR N.IDSISTEMA IS NULL))
                     AND ((N.CDALMOXARIFE = ISNULL(@P_CDALMOXARIFE, N.CDALMOXARIFE)) OR @P_CDALMOXARIFE = 'T')
                     AND ((N.CDLOCALESTOQ = ISNULL(@P_CDLOCALESTOQ, N.CDLOCALESTOQ)) OR @P_CDLOCALESTOQ = 'T')
                     AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                     AND N.CDTIPOOPER = T.CDTIPOOPER
                     AND T.IDCOMPEMISRE = 'C'
                     AND N.CDFILIEMIT = F.CDFILIAL
                     AND @P_CDFILIAL = I.CDFILIAL
                     AND (EXISTS (SELECT CDFILIAL
                                    FROM FILIAL FL
                                   WHERE ISNULL(CDFILIFISC,FL.CDFILIAL) = @P_CDFILIAL
                                     AND N.CDFILIEMIT = FL.CDFILIAL));
               OPEN C_NOTAFISCALSC;
               FETCH NEXT FROM C_NOTAFISCALSC INTO @V_NSC_CDFILIAL,   @V_NSC_NRLANCTONF, @V_NSC_CDMODDOCFISC,
                                                   @V_NSC_CDTIPOOPER, @V_NSC_CDFORNECED, @V_NSC_CDCLIENTE;
               WHILE (@@FETCH_STATUS = 0)
               BEGIN
                  SET @V_IDCAPACORRES = 'S';

                  IF (NOT @V_CDMODEDOCU = '' AND @V_IDCAPACORRES = 'S') BEGIN
                     IF CHARINDEX(@V_NSC_CDMODDOCFISC, @V_CDMODEDOCU) = 0 BEGIN
                        SET @V_IDCAPACORRES = 'N';
                     END;
                  END;

                  IF (NOT @V_CDTIPOOPER = '' AND @V_IDCAPACORRES = 'S') BEGIN
                     IF CHARINDEX(@V_NSC_CDTIPOOPER, @V_CDTIPOOPER) = 0 BEGIN
                        SET @V_IDCAPACORRES = 'N';
                     END;
                  END;

                  IF (NOT @V_CDFORNECED = '' AND @V_IDCAPACORRES = 'S') OR
                     (NOT @V_CDCLIENTE = '' AND @V_IDCAPACORRES = 'S')  BEGIN
                     IF (CHARINDEX(@V_NSC_CDFORNECED, @V_CDFORNECED) = 0) AND
                        (CHARINDEX(@V_NSC_CDCLIENTE, @V_CDCLIENTE) = 0)   BEGIN
                        SET @V_IDCAPACORRES = 'N';
                     END;
                  END;

                  /* Se atende as regras de capa de documento conforme parametrizao (regras) */
                  IF @V_IDCAPACORRES = 'S' BEGIN
                     /* Verifica se j possui RE (Capa de documento) gerada para o dia (RE CENTRALIZADORA) */
                     SET @V_NUMERORE = NULL;

                     SELECT @V_NUMERORE = MAX(N.NRSEQCAPADOC)
                       FROM NOTAFISCAL N, TIPOOPERAC T
                      WHERE N.IDENTRSAIDOP = 'S'
                        AND IDSTATUS IN ('A','E')
                        AND N.DTAPROVACAO   = @P_DATARE
                        AND ((N.IDSISTEMA IN ('EST','DFS','FAI','PDV') OR N.IDSISTEMA IS NULL))
                        AND NOT N.NRSEQCAPADOC IS NULL
                        AND N.CDPARAMEMISRE = @V_C_CDPARAMEMISRE
                        AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                        AND N.CDTIPOOPER = T.CDTIPOOPER
                        AND T.IDCOMPEMISRE = 'C'
                        AND (EXISTS (SELECT CDFILIAL
                                       FROM FILIAL FL
                                      WHERE ISNULL(CDFILIFISC,FL.CDFILIAL) = @P_CDFILIAL
                                        AND N.CDFILIEMIT = FL.CDFILIAL));

                     IF @V_NUMERORE IS NULL BEGIN
                        IF @V_IDSEQCRIADO <> 'S' BEGIN
                           IF ((@V_IDREINICIANUMRE = 'S') AND
                               (@P_DATARE > @V_DTFECMES)   AND
                               (@V_NRMAIORRE IS NULL))    BEGIN

                              /* Verifica se j possui RE (Capa de documento) gerada para o dia (RE CENTRALIZADORA) independente de regras */
                              SET @V_NUMERORED = NULL;

                              SELECT @V_NUMERORED = MAX(N.NRSEQCAPADOC)
                                FROM NOTAFISCAL N, TIPOOPERAC T
                               WHERE N.IDENTRSAIDOP = 'S'
                                 AND IDSTATUS IN ('A','E')
                                 AND N.DTAPROVACAO   = @P_DATARE
                                 AND ((N.IDSISTEMA IN ('EST','DFS','FAI','PDV') OR N.IDSISTEMA IS NULL))
                                 AND NOT N.NRSEQCAPADOC IS NULL
                                 AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                                 AND N.CDTIPOOPER = T.CDTIPOOPER
                                 AND T.IDCOMPEMISRE = 'C'
                                 AND (EXISTS (SELECT CDFILIAL
                                                FROM FILIAL FL
                                               WHERE ISNULL(CDFILIFISC,FL.CDFILIAL) = @P_CDFILIAL
                                                 AND N.CDFILIEMIT = FL.CDFILIAL));

                              IF @V_NUMERORED IS NULL BEGIN
                                 UPDATE NOVOCODIGO
                                    SET NRSEQUENCIAL = RIGHT(REPLICATE('0',20) + '0',20)
                                  WHERE CDCONTADOR = 'RENOTAFISCAL' + @P_CDFILIAL;
                              END;
                           END;

                           SET @V_PRXNUMRE = NULL;

                           SET @V_NOVO_CODIGO = 'RENOTAFISCAL'+@P_CDFILIAL;
                           EXEC NOVO_CODIGO @V_NOVO_CODIGO, @P_SEQUENCIAL = @V_PRXNUMRE OUTPUT, @P_QTDE = 1;
                           SET @V_NUMERORE = SUBSTRING(@V_PRXNUMRE, 11, 10);

                           /* Insere novo nmero de RE */
                           INSERT INTO NUMERORNF
                             (CDFILIAL, DTNUMERNF, IDENTRSAID, IDTIPOOPERNF, NRNUMERNF)
                           VALUES
                             (@P_CDFILIAL, @P_DATARE, @P_IDENTRSAID, @P_IDENTRSAID, @V_NUMERORE);

                           UPDATE PARAMFILIAL
                              SET NRRENOTAFISC = @V_NUMERORE
                            WHERE CDFILIAL = @P_CDFILIAL;

                           SET @V_IDSEQCRIADO = 'S';
                        END;
                     END;
                     UPDATE NOTAFISCAL
                        SET CDPARAMEMISRE = @V_C_CDPARAMEMISRE,
                            NRSEQCAPADOC = @V_NUMERORE
                      WHERE CDFILIAL = @V_NSC_CDFILIAL
                        AND NRLANCTONF = @V_NSC_NRLANCTONF;
                  END;
                  FETCH NEXT FROM C_NOTAFISCALSC INTO @V_NSC_CDFILIAL,   @V_NSC_NRLANCTONF, @V_NSC_CDMODDOCFISC,
                                                     @V_NSC_CDTIPOOPER,  @V_NSC_CDFORNECED, @V_NSC_CDCLIENTE;
               END;
               CLOSE C_NOTAFISCALSC;
               DEALLOCATE C_NOTAFISCALSC;

               FETCH NEXT FROM C_PARAMEMIS INTO @V_C_CDPARAMEMISRE, @V_C_DSOBPARAMEMISRE;
            END;
            CLOSE C_PARAMEMIS;
            DEALLOCATE C_PARAMEMIS;

            /*   Verifica notas que no possuem nenhuma regra e gera uma capa de documento (maior capa do dia) */
            SET @V_IDSEQCRIADO = 'N';

            /* Notas fiscais de sada - RE Centralizadora */
            DECLARE C_NOTAFISCALSC CURSOR FOR
               SELECT DISTINCT N.CDFILIAL, N.NRLANCTONF, N.CDMODDOCFISC, N.CDTIPOOPER, ISNULL(N.CDFORNECED,''), ISNULL(N.CDCLIENTE,'')
                 FROM FILIAL F, FILIAL I, TIPOOPERAC T,
                      NOTAFISCAL N LEFT JOIN ALMOXARIFADO A ON (N.CDALMOXARIFE = A.CDALMOXARIFE) AND
                                                                (N.CDFILIAL = A.CDFILIAL)
                                   LEFT JOIN LOCALESTOQUE L ON (N.CDALMOXARIFE = L.CDALMOXARIFE) AND
                                                               (N.CDFILIAL = L.CDFILIAL)         AND
                                                               (N.CDLOCALESTOQ = L.CDLOCALESTOQ)
                WHERE N.IDENTRSAIDOP = 'S'
                  AND IDSTATUS IN ('A','E')
                  AND N.NRSEQCAPADOC IS NULL
                  AND N.DTAPROVACAO   = @P_DATARE
                  AND ((N.IDSISTEMA IN ('EST','DFS','FAI','PDV') OR N.IDSISTEMA IS NULL))
                  AND ((N.CDALMOXARIFE = ISNULL(@P_CDALMOXARIFE, N.CDALMOXARIFE)) OR @P_CDALMOXARIFE = 'T')
                  AND ((N.CDLOCALESTOQ = ISNULL(@P_CDLOCALESTOQ, N.CDLOCALESTOQ)) OR @P_CDLOCALESTOQ = 'T')
                  AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                  AND N.CDTIPOOPER = T.CDTIPOOPER
                  AND T.IDCOMPEMISRE = 'C'
                  AND N.CDFILIEMIT = F.CDFILIAL
                  AND @P_CDFILIAL = I.CDFILIAL
                  AND (EXISTS (SELECT CDFILIAL
                                 FROM FILIAL FL
                                WHERE ISNULL(CDFILIFISC,FL.CDFILIAL) = @P_CDFILIAL
                                  AND N.CDFILIEMIT = FL.CDFILIAL));
            OPEN C_NOTAFISCALSC;
            FETCH NEXT FROM C_NOTAFISCALSC INTO @V_NSC_CDFILIAL,   @V_NSC_NRLANCTONF, @V_NSC_CDMODDOCFISC,
                                                @V_NSC_CDTIPOOPER, @V_NSC_CDFORNECED, @V_NSC_CDCLIENTE;

            WHILE (@@FETCH_STATUS = 0)
            BEGIN
               IF @V_IDSEQCRIADO <> 'S' BEGIN
                  IF ((@V_IDREINICIANUMRE = 'S') AND
                      (@P_DATARE > @V_DTFECMES)   AND
                      (@V_NRMAIORRE IS NULL))    BEGIN

                     /* Verifica se j possui RE (Capa de documento) gerada para o dia */
                     SELECT @V_NUMERORE = MAX(NRNUMERNF)
                       FROM NUMERORNF
                      WHERE CDFILIAL = @P_CDFILIAL
                        AND DTNUMERNF = @P_DATARE
                        AND IDENTRSAID = 'S'
                        AND IDTIPOOPERNF = 'S';

                     IF @V_NUMERORE IS NULL BEGIN
                        UPDATE NOVOCODIGO
                           SET NRSEQUENCIAL = RIGHT(REPLICATE('0',20) + '0',20)
                         WHERE CDCONTADOR = 'RENOTAFISCAL' + @P_CDFILIAL;
                     END;
                  END;

                  SET @V_PRXNUMRE = NULL;

                  SET @V_NOVO_CODIGO = 'RENOTAFISCAL'+@P_CDFILIAL;
                  EXEC NOVO_CODIGO @V_NOVO_CODIGO, @P_SEQUENCIAL = @V_PRXNUMRE OUTPUT, @P_QTDE = 1;
                  SET @V_NUMERORE = SUBSTRING(@V_PRXNUMRE, 11, 10);

                  /* Insere novo nmero de RE */
                  INSERT INTO NUMERORNF
                    (CDFILIAL, DTNUMERNF, IDENTRSAID, IDTIPOOPERNF, NRNUMERNF)
                  VALUES
                    (@P_CDFILIAL, @P_DATARE, @P_IDENTRSAID, @P_IDENTRSAID, @V_NUMERORE);

                  UPDATE PARAMFILIAL
                     SET NRRENOTAFISC = @V_NUMERORE
                   WHERE CDFILIAL = @P_CDFILIAL;

                  SET @V_IDSEQCRIADO = 'S';
               END;
               UPDATE NOTAFISCAL
                  SET NRSEQCAPADOC = @V_NUMERORE
                WHERE CDFILIAL = @V_NSC_CDFILIAL
                  AND NRLANCTONF = @V_NSC_NRLANCTONF;

               FETCH NEXT FROM C_NOTAFISCALSC INTO @V_NSC_CDFILIAL,   @V_NSC_NRLANCTONF, @V_NSC_CDMODDOCFISC,
                                                   @V_NSC_CDTIPOOPER, @V_NSC_CDFORNECED, @V_NSC_CDCLIENTE;
            END;
            CLOSE C_NOTAFISCALSC;
            DEALLOCATE C_NOTAFISCALSC;
         END
         ELSE BEGIN
            /* Verifica se j possui RE (Capa de documento) gerada para o dia (RE CENTRALIZADORA) */
            SET @V_NUMERORE = NULL;

            SELECT @V_NUMERORE = MAX(N.NRSEQCAPADOC)
              FROM NOTAFISCAL N, TIPOOPERAC T
             WHERE N.IDENTRSAIDOP = 'S'
               AND IDSTATUS IN ('A','E')
               AND N.DTAPROVACAO   = @P_DATARE
               AND ((N.IDSISTEMA IN ('EST','DFS','FAI','PDV') OR N.IDSISTEMA IS NULL))
               AND NOT N.NRSEQCAPADOC IS NULL
               AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
               AND N.CDTIPOOPER = T.CDTIPOOPER
               AND T.IDCOMPEMISRE = 'C'
               AND (EXISTS (SELECT CDFILIAL
                              FROM FILIAL FL
                             WHERE ISNULL(CDFILIFISC,FL.CDFILIAL) = @P_CDFILIAL
                               AND N.CDFILIEMIT = FL.CDFILIAL));


            SET @V_IDSEQCRIADO = 'N';

            DECLARE C_NOTAFISCALSC CURSOR FOR
               SELECT DISTINCT N.CDFILIAL, N.NRLANCTONF, N.CDMODDOCFISC, N.CDTIPOOPER, ISNULL(N.CDFORNECED,''), ISNULL(N.CDCLIENTE,'')
                 FROM FILIAL F, FILIAL I, TIPOOPERAC T,
                      NOTAFISCAL N LEFT JOIN ALMOXARIFADO A ON (N.CDALMOXARIFE = A.CDALMOXARIFE) AND
                                                                (N.CDFILIAL = A.CDFILIAL)
                                   LEFT JOIN LOCALESTOQUE L ON (N.CDALMOXARIFE = L.CDALMOXARIFE) AND
                                                               (N.CDFILIAL = L.CDFILIAL)         AND
                                                               (N.CDLOCALESTOQ = L.CDLOCALESTOQ)
                WHERE N.IDENTRSAIDOP = 'S'
                  AND IDSTATUS IN ('A','E')
                  AND N.DTAPROVACAO   = @P_DATARE
                  AND N.NRSEQCAPADOC IS NULL
                  AND ((N.IDSISTEMA IN ('EST','DFS','FAI','PDV') OR N.IDSISTEMA IS NULL))
                  AND ((N.CDALMOXARIFE = ISNULL(@P_CDALMOXARIFE, N.CDALMOXARIFE)) OR @P_CDALMOXARIFE = 'T')
                  AND ((N.CDLOCALESTOQ = ISNULL(@P_CDLOCALESTOQ, N.CDLOCALESTOQ)) OR @P_CDLOCALESTOQ = 'T')
                  AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
                  AND N.CDTIPOOPER = T.CDTIPOOPER
                  AND T.IDCOMPEMISRE = 'C'
                  AND N.CDFILIEMIT = F.CDFILIAL
                  AND @P_CDFILIAL = I.CDFILIAL
                  AND (EXISTS (SELECT CDFILIAL
                                 FROM FILIAL FL
                                WHERE ISNULL(CDFILIFISC,FL.CDFILIAL) = @P_CDFILIAL
                                  AND N.CDFILIEMIT = FL.CDFILIAL));
            OPEN C_NOTAFISCALSC;
            FETCH NEXT FROM C_NOTAFISCALSC INTO @V_NSC_CDFILIAL,   @V_NSC_NRLANCTONF, @V_NSC_CDMODDOCFISC,
                                                @V_NSC_CDTIPOOPER, @V_NSC_CDFORNECED, @V_NSC_CDCLIENTE;
            WHILE (@@FETCH_STATUS = 0)
            BEGIN
               IF @V_NUMERORE IS NULL BEGIN
                  IF @V_IDSEQCRIADO <> 'S' BEGIN
                     IF ((@V_IDREINICIANUMRE = 'S') AND
                         (@P_DATARE > @V_DTFECMES)   AND
                         (@V_NRMAIORRE IS NULL))    BEGIN
                        UPDATE NOVOCODIGO
                           SET NRSEQUENCIAL = RIGHT(REPLICATE('0',20) + '0',20)
                         WHERE CDCONTADOR = 'RENOTAFISCAL' + @P_CDFILIAL;
                     END;

                     SET @V_PRXNUMRE = NULL;

                     SET @V_NOVO_CODIGO = 'RENOTAFISCAL'+@P_CDFILIAL;
                     EXEC NOVO_CODIGO @V_NOVO_CODIGO, @P_SEQUENCIAL = @V_PRXNUMRE OUTPUT, @P_QTDE = 1;
                     SET @V_NUMERORE = SUBSTRING(@V_PRXNUMRE, 11, 10);

                     /* Insere novo nmero de RE */
                     INSERT INTO NUMERORNF
                       (CDFILIAL, DTNUMERNF, IDENTRSAID, IDTIPOOPERNF, NRNUMERNF)
                     VALUES
                       (@P_CDFILIAL, @P_DATARE, @P_IDENTRSAID, @P_IDENTRSAID, @V_NUMERORE);

                     UPDATE PARAMFILIAL
                        SET NRRENOTAFISC = @V_NUMERORE
                      WHERE CDFILIAL = @P_CDFILIAL;

                     SET @V_IDSEQCRIADO = 'S';
                  END;
               END;

               UPDATE NOTAFISCAL
                  SET NRSEQCAPADOC = @V_NUMERORE
                WHERE CDFILIAL = @V_NSC_CDFILIAL
                  AND NRLANCTONF = @V_NSC_NRLANCTONF;

               FETCH NEXT FROM C_NOTAFISCALSC INTO @V_NSC_CDFILIAL,   @V_NSC_NRLANCTONF, @V_NSC_CDMODDOCFISC,
                                                   @V_NSC_CDTIPOOPER, @V_NSC_CDFORNECED, @V_NSC_CDCLIENTE;
            END;
            CLOSE C_NOTAFISCALSC;
            DEALLOCATE C_NOTAFISCALSC;
         END;
      END;
   END;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'GERA_RE','5.75.XXX';
GO
